@@ -252,10 +252,10 @@ static size_t *
 rand_sizes(size_t min, size_t max, size_t n_ops)
 {
 	assert(n_ops != 0);
-	size_t *rand_sizes = (size_t *)malloc(n_ops * sizeof(size_t));
-	if (rand_sizes == NULL) {
+	auto *rand_sizes = (size_t *)malloc(n_ops * sizeof(size_t));
+	if (rand_sizes == nullptr) {
 		perror("malloc");
-		return NULL;
+		return nullptr;
 	}
 	for (size_t i = 0; i < n_ops; i++) {
 		rand_sizes[i] = RRAND(max, min);
@@ -273,7 +273,7 @@ random_types(struct pobj_bench *bench_priv, struct benchmark_args *args)
 	assert(bench_priv->args_priv->n_objs != 0);
 	bench_priv->random_types = (size_t *)malloc(
 		bench_priv->args_priv->n_objs * sizeof(size_t));
-	if (bench_priv->random_types == NULL) {
+	if (bench_priv->random_types == nullptr) {
 		perror("malloc");
 		return -1;
 	}
@@ -293,16 +293,23 @@ pobj_init(struct benchmark *bench, struct benchmark_args *args)
 	size_t psize;
 	size_t n_objs;
 
-	assert(bench != NULL);
-	assert(args != NULL);
+	assert(bench != nullptr);
+	assert(args != nullptr);
 
-	struct pobj_bench *bench_priv =
+	enum file_type type = util_file_get_type(args->fname);
+	if (type == OTHER_ERROR) {
+		fprintf(stderr, "could not check type of file %s\n",
+			args->fname);
+		return -1;
+	}
+
+	auto *bench_priv =
 		(struct pobj_bench *)malloc(sizeof(struct pobj_bench));
-	if (bench_priv == NULL) {
+	if (bench_priv == nullptr) {
 		perror("malloc");
 		return -1;
 	}
-	assert(args->opts != NULL);
+	assert(args->opts != nullptr);
 
 	bench_priv->args_priv = (struct pobj_args *)args->opts;
 	bench_priv->args_priv->obj_size = args->dsize;
@@ -313,9 +320,11 @@ pobj_init(struct benchmark *bench, struct benchmark_args *args)
 	bench_priv->pool = bench_priv->n_pools > 1 ? diff_num : one_num;
 	bench_priv->obj = !bench_priv->args_priv->one_obj ? diff_num : one_num;
 
-	if (args->is_poolset && bench_priv->n_pools > 1) {
-		fprintf(stderr, "cannot use poolset for multiple pools,"
-				" please use -P|--one-pool option instead");
+	if ((args->is_poolset || type == TYPE_DEVDAX) &&
+	    bench_priv->n_pools > 1) {
+		fprintf(stderr,
+			"cannot use poolset nor device dax for multiple pools,"
+			" please use -P|--one-pool option instead");
 		goto free_bench_priv;
 	}
 	/*
@@ -326,9 +335,8 @@ pobj_init(struct benchmark *bench, struct benchmark_args *args)
 	n_objs = bench_priv->args_priv->n_objs;
 	if (bench_priv->n_pools == 1)
 		n_objs *= args->n_threads;
-	psize = n_objs * args->dsize * args->n_threads * FACTOR;
-	if (psize < PMEMOBJ_MIN_POOL)
-		psize = PMEMOBJ_MIN_POOL;
+	psize = PMEMOBJ_MIN_POOL +
+		n_objs * args->dsize * args->n_threads * FACTOR;
 
 	/* assign type_number determining function */
 	bench_priv->type_mode =
@@ -342,14 +350,14 @@ pobj_init(struct benchmark *bench, struct benchmark_args *args)
 				goto free_bench_priv;
 			break;
 		default:
-			bench_priv->random_types = NULL;
+			bench_priv->random_types = nullptr;
 	}
 	bench_priv->fn_type_num = type_mode_func[bench_priv->type_mode];
 
 	/* assign size determining function */
 	bench_priv->fn_size =
 		bench_priv->args_priv->range ? range_size : static_size;
-	bench_priv->rand_sizes = NULL;
+	bench_priv->rand_sizes = nullptr;
 	if (bench_priv->args_priv->range) {
 		if (bench_priv->args_priv->min_size > args->dsize) {
 			fprintf(stderr, "Invalid allocation size");
@@ -359,21 +367,21 @@ pobj_init(struct benchmark *bench, struct benchmark_args *args)
 			rand_sizes(bench_priv->args_priv->min_size,
 				   bench_priv->args_priv->obj_size,
 				   bench_priv->args_priv->n_objs);
-		if (bench_priv->rand_sizes == NULL)
+		if (bench_priv->rand_sizes == nullptr)
 			goto free_random_types;
 	}
 
 	assert(bench_priv->n_pools > 0);
 	bench_priv->pop = (PMEMobjpool **)calloc(bench_priv->n_pools,
 						 sizeof(PMEMobjpool *));
-	if (bench_priv->pop == NULL) {
+	if (bench_priv->pop == nullptr) {
 		perror("calloc");
 		goto free_random_sizes;
 	}
 
 	bench_priv->sets = (const char **)calloc(bench_priv->n_pools,
 						 sizeof(const char *));
-	if (bench_priv->sets == NULL) {
+	if (bench_priv->sets == nullptr) {
 		perror("calloc");
 		goto free_pop;
 	}
@@ -388,7 +396,7 @@ pobj_init(struct benchmark *bench, struct benchmark_args *args)
 		for (i = 0; i < bench_priv->n_pools; i++) {
 			bench_priv->sets[i] =
 				(char *)malloc(path_len * sizeof(char));
-			if (bench_priv->sets[i] == NULL) {
+			if (bench_priv->sets[i] == nullptr) {
 				perror("malloc");
 				goto free_sets;
 			}
@@ -402,24 +410,23 @@ pobj_init(struct benchmark *bench, struct benchmark_args *args)
 			bench_priv->pop[i] =
 				pmemobj_create(bench_priv->sets[i], LAYOUT_NAME,
 					       psize, FILE_MODE);
-			if (bench_priv->pop[i] == NULL) {
+			if (bench_priv->pop[i] == nullptr) {
 				perror(pmemobj_errormsg());
 				goto free_sets;
 			}
 		}
 	} else {
-		if (args->is_poolset) {
+		if (args->is_poolset || type == TYPE_DEVDAX) {
 			if (args->fsize < psize) {
-				fprintf(stderr, "insufficient size of "
-						"poolset\n");
+				fprintf(stderr, "file size too large\n");
 				goto free_pools;
 			}
 			psize = 0;
 		}
-		bench_priv->sets[0] = (const char *)args->fname;
+		bench_priv->sets[0] = args->fname;
 		bench_priv->pop[0] = pmemobj_create(
 			bench_priv->sets[0], LAYOUT_NAME, psize, FILE_MODE);
-		if (bench_priv->pop[0] == NULL) {
+		if (bench_priv->pop[0] == nullptr) {
 			perror(pmemobj_errormsg());
 			goto free_pools;
 		}
@@ -452,7 +459,7 @@ free_bench_priv:
 static int
 pobj_direct_init(struct benchmark *bench, struct benchmark_args *args)
 {
-	struct pobj_args *pa = (struct pobj_args *)args->opts;
+	auto *pa = (struct pobj_args *)args->opts;
 	pa->n_objs = pa->one_obj ? 1 : args->n_ops_per_thread;
 	if (pobj_init(bench, args) != 0)
 		return -1;
@@ -466,8 +473,7 @@ static int
 pobj_exit(struct benchmark *bench, struct benchmark_args *args)
 {
 	size_t i;
-	struct pobj_bench *bench_priv =
-		(struct pobj_bench *)pmembench_get_priv(bench);
+	auto *bench_priv = (struct pobj_bench *)pmembench_get_priv(bench);
 	if (bench_priv->n_pools > 1) {
 		for (i = 0; i < bench_priv->n_pools; i++) {
 			pmemobj_close(bench_priv->pop[i]);
@@ -492,11 +498,9 @@ pobj_init_worker(struct benchmark *bench, struct benchmark_args *args,
 		 struct worker_info *worker)
 {
 	size_t i, idx = worker->index;
-	struct pobj_bench *bench_priv =
-		(struct pobj_bench *)pmembench_get_priv(bench);
-	struct pobj_worker *pw =
-		(struct pobj_worker *)calloc(1, sizeof(struct pobj_worker));
-	if (pw == NULL) {
+	auto *bench_priv = (struct pobj_bench *)pmembench_get_priv(bench);
+	auto *pw = (struct pobj_worker *)calloc(1, sizeof(struct pobj_worker));
+	if (pw == nullptr) {
 		perror("calloc");
 		return -1;
 	}
@@ -504,7 +508,7 @@ pobj_init_worker(struct benchmark *bench, struct benchmark_args *args,
 	worker->priv = pw;
 	pw->oids = (PMEMoid *)calloc(bench_priv->args_priv->n_objs,
 				     sizeof(PMEMoid));
-	if (pw->oids == NULL) {
+	if (pw->oids == nullptr) {
 		free(pw);
 		perror("calloc");
 		return -1;
@@ -514,8 +518,8 @@ pobj_init_worker(struct benchmark *bench, struct benchmark_args *args,
 	for (i = 0; i < bench_priv->args_priv->n_objs; i++) {
 		size_t size = bench_priv->fn_size(bench_priv, i);
 		size_t type = bench_priv->fn_type_num(bench_priv, idx, i);
-		if (pmemobj_alloc(pop, &pw->oids[i], size, type, NULL, NULL) !=
-		    0) {
+		if (pmemobj_alloc(pop, &pw->oids[i], size, type, nullptr,
+				  nullptr) != 0) {
 			perror("pmemobj_alloc");
 			goto out;
 		}
@@ -535,13 +539,24 @@ out:
 static int
 pobj_direct_op(struct benchmark *bench, struct operation_info *info)
 {
-	struct pobj_bench *bench_priv =
-		(struct pobj_bench *)pmembench_get_priv(bench);
-	struct pobj_worker *pw = (struct pobj_worker *)info->worker->priv;
+	auto *bench_priv = (struct pobj_bench *)pmembench_get_priv(bench);
+	auto *pw = (struct pobj_worker *)info->worker->priv;
 	size_t idx = bench_priv->obj(info->index);
-	if (pmemobj_direct(pw->oids[idx]) == NULL)
-		return -1;
+	/* Query an invalid uuid:off pair to invalidate the cache. */
+	PMEMoid bad = {1, 1};
+#define OBJ_DIRECT_NITER 1024
+	/*
+	 * As we measure a very fast operation, we need a loop inside the
+	 * test harness.
+	 */
+	for (int i = 0; i < OBJ_DIRECT_NITER; i++) {
+		if (pmemobj_direct(pw->oids[idx]) == nullptr)
+			return -1;
+		if (pmemobj_direct(bad) != nullptr)
+			return -1;
+	}
 	return 0;
+#undef OBJ_DIRECT_NITER
 }
 
 /*
@@ -550,12 +565,11 @@ pobj_direct_op(struct benchmark *bench, struct operation_info *info)
 static int
 pobj_open_op(struct benchmark *bench, struct operation_info *info)
 {
-	struct pobj_bench *bench_priv =
-		(struct pobj_bench *)pmembench_get_priv(bench);
+	auto *bench_priv = (struct pobj_bench *)pmembench_get_priv(bench);
 	size_t idx = bench_priv->pool(info->worker->index);
 	pmemobj_close(bench_priv->pop[idx]);
 	bench_priv->pop[idx] = pmemobj_open(bench_priv->sets[idx], LAYOUT_NAME);
-	if (bench_priv->pop[idx] == NULL)
+	if (bench_priv->pop[idx] == nullptr)
 		return -1;
 	return 0;
 }
@@ -567,9 +581,8 @@ static void
 pobj_free_worker(struct benchmark *bench, struct benchmark_args *args,
 		 struct worker_info *worker)
 {
-	struct pobj_worker *pw = (struct pobj_worker *)worker->priv;
-	struct pobj_bench *bench_priv =
-		(struct pobj_bench *)pmembench_get_priv(bench);
+	auto *pw = (struct pobj_worker *)worker->priv;
+	auto *bench_priv = (struct pobj_bench *)pmembench_get_priv(bench);
 	for (size_t i = 0; i < bench_priv->args_priv->n_objs; i++)
 		pmemobj_free(&pw->oids[i]);
 	free(pw->oids);
@@ -584,9 +597,9 @@ static struct benchmark_clo pobj_direct_clo[4];
 
 static struct benchmark_clo pobj_open_clo[3];
 
-CONSTRUCTOR(pmemobj_gen_costructor)
+CONSTRUCTOR(pmemobj_gen_constructor)
 void
-pmemobj_gen_costructor(void)
+pmemobj_gen_constructor(void)
 {
 	pobj_direct_clo[0].opt_short = 'T';
 	pobj_direct_clo[0].opt_long = "type-number";
