@@ -1,5 +1,5 @@
 /*
- * Copyright 2015-2017, Intel Corporation
+ * Copyright 2015-2019, Intel Corporation
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -113,24 +113,24 @@ struct {\
 } while (0)
 
 #define PDLL_FOREACH(entry, head, field)\
-for (entry = ((head)).first; !TOID_IS_NULL(entry);\
-		entry = D_RO(entry)->field.next)
+for ((entry) = ((head)).first; !TOID_IS_NULL(entry);\
+		(entry) = D_RO(entry)->field.next)
 
 #define PDLL_FOREACH_SAFE(entry, next, head, field)\
-for (entry = ((head)).first; !TOID_IS_NULL(entry) &&\
-		(next = D_RO(entry)->field.next, 1);\
-		entry = next)
+for ((entry) = ((head)).first; !TOID_IS_NULL(entry) &&\
+		((next) = D_RO(entry)->field.next, 1);\
+		(entry) = (next))
 
 #define PDLL_INSERT_HEAD(head, entry, field) do {\
-	pmemobj_tx_add_range_direct(&head.first, sizeof(head.first));\
+	pmemobj_tx_add_range_direct(&(head).first, sizeof((head).first));\
 	TX_ADD_FIELD(entry, field);\
-	D_RW(entry)->field.next = head.first;\
+	D_RW(entry)->field.next = (head).first;\
 	D_RW(entry)->field.prev =\
 		(typeof(D_RW(entry)->field.prev))OID_NULL;\
-	head.first = entry;\
-	if (TOID_IS_NULL(head.last)) {\
-		pmemobj_tx_add_range_direct(&head.last, sizeof(head.last));\
-		head.last = entry;\
+	(head).first = entry;\
+	if (TOID_IS_NULL((head).last)) {\
+		pmemobj_tx_add_range_direct(&(head).last, sizeof((head).last));\
+		(head).last = entry;\
 	}\
 	typeof(entry) next = D_RO(entry)->field.next;\
 	if (!TOID_IS_NULL(next)) {\
@@ -141,25 +141,27 @@ for (entry = ((head)).first; !TOID_IS_NULL(entry) &&\
 } while (0)
 
 #define PDLL_REMOVE(head, entry, field) do {\
-	if (TOID_EQUALS(head.first, entry) &&\
-		TOID_EQUALS(head.last, entry)) {\
-		pmemobj_tx_add_range_direct(&head.first, sizeof(head.first));\
-		pmemobj_tx_add_range_direct(&head.last, sizeof(head.last));\
-		head.first = (typeof(D_RW(entry)->field.prev))OID_NULL;\
-		head.last = (typeof(D_RW(entry)->field.prev))OID_NULL;\
-	} else if (TOID_EQUALS(head.first, entry)) {\
+	if (TOID_EQUALS((head).first, entry) &&\
+		TOID_EQUALS((head).last, entry)) {\
+		pmemobj_tx_add_range_direct(&(head).first,\
+				sizeof((head).first));\
+		pmemobj_tx_add_range_direct(&(head).last, sizeof((head).last));\
+		(head).first = (typeof(D_RW(entry)->field.prev))OID_NULL;\
+		(head).last = (typeof(D_RW(entry)->field.prev))OID_NULL;\
+	} else if (TOID_EQUALS((head).first, entry)) {\
 		typeof(entry) next = D_RW(entry)->field.next;\
 		pmemobj_tx_add_range_direct(&D_RW(next)->field.prev,\
 				sizeof(D_RW(next)->field.prev));\
-		pmemobj_tx_add_range_direct(&head.first, sizeof(head.first));\
-		head.first = D_RO(entry)->field.next;\
+		pmemobj_tx_add_range_direct(&(head).first,\
+				sizeof((head).first));\
+		(head).first = D_RO(entry)->field.next;\
 		D_RW(next)->field.prev.oid = OID_NULL;\
-	} else if (TOID_EQUALS(head.last, entry)) {\
+	} else if (TOID_EQUALS((head).last, entry)) {\
 		typeof(entry) prev = D_RW(entry)->field.prev;\
 		pmemobj_tx_add_range_direct(&D_RW(prev)->field.next,\
 				sizeof(D_RW(prev)->field.next));\
-		pmemobj_tx_add_range_direct(&head.last, sizeof(head.last));\
-		head.last = D_RO(entry)->field.prev;\
+		pmemobj_tx_add_range_direct(&(head).last, sizeof((head).last));\
+		(head).last = D_RO(entry)->field.prev;\
 		D_RW(prev)->field.next.oid = OID_NULL;\
 	} else {\
 		typeof(entry) prev = D_RW(entry)->field.prev;\
@@ -759,7 +761,6 @@ pmemobjfs_read(struct pmemobjfs *objfs, TOID(struct objfs_inode) inode,
 			memcpy(buff, &D_RW(block)[block_off], block_size);
 		}
 
-
 		buff += block_size;
 		off += block_size;
 		sz -= block_size;
@@ -914,7 +915,6 @@ pmemobjfs_remove_dir_entry_name(struct pmemobjfs *objfs,
 	} TX_END
 }
 
-
 /*
  * pmemobjfs_add_dir_entry -- add new directory entry
  */
@@ -1029,7 +1029,6 @@ pmemobjfs_mkdir(struct pmemobjfs *objfs, TOID(struct objfs_inode) inode,
 	return ret;
 }
 
-
 /*
  * pmemobjfs_remove_dir -- remove directory from directory
  */
@@ -1776,7 +1775,7 @@ static int
 pmemobjfs_fuse_write(const char *path, const char *buff, size_t size,
 		off_t offset, struct fuse_file_info *fi)
 {
-	log("%s size = %lu off = %lu", path, size, offset);
+	log("%s size = %zu off = %lu", path, size, offset);
 	struct pmemobjfs *objfs = PMEMOBJFS;
 
 	if (!fi->fh)
@@ -1799,7 +1798,7 @@ static int
 pmemobjfs_fuse_read(const char *path, char *buff, size_t size, off_t off,
 		struct fuse_file_info *fi)
 {
-	log("%s size = %lu off = %lu", path, size, off);
+	log("%s size = %zu off = %lu", path, size, off);
 	struct pmemobjfs *objfs = PMEMOBJFS;
 
 	if (!fi->fh)
@@ -2345,7 +2344,7 @@ pmemobjfs_mkfs_main(int argc, char *argv[])
 	}
 
 	if (bsize < PMEMOBJFS_MIN_BLOCK_SIZE) {
-		fprintf(stderr, "error: minimum block size is %lu\n",
+		fprintf(stderr, "error: minimum block size is %zu\n",
 				PMEMOBJFS_MIN_BLOCK_SIZE);
 		return -1;
 	}
@@ -2375,8 +2374,8 @@ pmemobjfs_tx_ioctl(const char *dir, int req)
 	if (!path)
 		return -1;
 
-	strncpy(path, dir, dirlen);
-	strncat(path, PMEMOBJFS_TMP_TEMPLATE, tmpllen);
+	memcpy(path, dir, dirlen);
+	strcpy(path + dirlen, PMEMOBJFS_TMP_TEMPLATE);
 
 	/* create temporary file */
 	mode_t prev_umask = umask(S_IRWXG | S_IRWXO);
@@ -2445,7 +2444,6 @@ main(int argc, char *argv[])
 	log("\n\n\nPMEMOBJFS\n");
 #endif
 
-
 	const char *fname = argv[argc - 2];
 	struct pmemobjfs *objfs = calloc(1, sizeof(*objfs));
 	if (!objfs) {
