@@ -1,5 +1,5 @@
 /*
- * Copyright 2015-2017, Intel Corporation
+ * Copyright 2015-2019, Intel Corporation
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -49,7 +49,6 @@ struct tree_map_node_item {
 	PMEMoid value;
 };
 
-
 struct tree_map_node {
 	int n; /* number of occupied slots */
 	struct tree_map_node_item items[BTREE_ORDER - 1];
@@ -101,7 +100,6 @@ btree_map_clear_node(TOID(struct tree_map_node) node)
 	TX_FREE(node);
 }
 
-
 /*
  * btree_map_clear -- removes all elements from the map
  */
@@ -122,7 +120,6 @@ btree_map_clear(PMEMobjpool *pop, TOID(struct btree_map) map)
 	return ret;
 }
 
-
 /*
  * btree_map_destroy -- cleanups and frees btree instance
  */
@@ -198,6 +195,7 @@ btree_map_create_split_node(TOID(struct tree_map_node) node,
 
 	int c = (BTREE_ORDER / 2);
 	*m = D_RO(node)->items[c - 1]; /* select median item */
+	TX_ADD(node);
 	set_empty_item(&D_RW(node)->items[c - 1]);
 
 	/* move everything right side of median to the new node */
@@ -364,7 +362,6 @@ btree_map_rotate_left(TOID(struct tree_map_node) lsb,
 	TX_ADD_FIELD(parent, items[p - 1]);
 	D_RW(parent)->items[p - 1] = D_RO(lsb)->items[D_RO(lsb)->n - 1];
 
-	TX_ADD(node);
 	/* rotate the node children */
 	memmove(D_RW(node)->slots + 1, D_RO(node)->slots,
 		sizeof(TOID(struct tree_map_node)) * (D_RO(node)->n));
@@ -495,10 +492,10 @@ btree_map_remove_from_node(TOID(struct btree_map) map,
 }
 
 #define NODE_CONTAINS_ITEM(_n, _i, _k)\
-(_i != D_RO(_n)->n && D_RO(_n)->items[_i].key == _k)
+((_i) != D_RO(_n)->n && D_RO(_n)->items[_i].key == (_k))
 
 #define NODE_CHILD_CAN_CONTAIN_ITEM(_n, _i, _k)\
-(_i == D_RO(_n)->n || D_RO(_n)->items[_i].key > _k) &&\
+((_i) == D_RO(_n)->n || D_RO(_n)->items[_i].key > (_k)) &&\
 !TOID_IS_NULL(D_RO(_n)->slots[_i])
 
 /*
