@@ -1,5 +1,5 @@
 /*
- * Copyright 2014-2018, Intel Corporation
+ * Copyright 2014-2019, Intel Corporation
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -205,9 +205,6 @@ pmem_drain(void)
 	LOG(15, NULL);
 
 	Funcs.predrain_fence();
-
-	VALGRIND_DO_COMMIT;
-	VALGRIND_DO_FENCE;
 }
 
 /*
@@ -275,9 +272,9 @@ pmem_msync(const void *addr, size_t len)
 	VALGRIND_DO_CHECK_MEM_IS_ADDRESSABLE(addr, len);
 
 	/*
-	 * msync requires len to be a multiple of pagesize, so
-	 * adjust addr and len to represent the full 4k chunks
-	 * covering the given range.
+	 * msync requires addr to be a multiple of pagesize but there are no
+	 * requirements for len. Align addr down and change len so that
+	 * [addr, addr + len) still contains initial range.
 	 */
 
 	/* increase len by the amount we gain when we round addr down */
@@ -307,13 +304,17 @@ pmem_msync(const void *addr, size_t len)
 }
 
 /*
- * is_pmem_always -- (internal) always true version of pmem_is_pmem()
+ * is_pmem_always -- (internal) always true (for meaningful parameters) version
+ *	of pmem_is_pmem()
  */
 static int
 is_pmem_always(const void *addr, size_t len)
 {
 	LOG(3, "addr %p len %zu", addr, len);
 
+	if (len == 0)
+		return 0;
+
 	return 1;
 }
 
@@ -421,7 +422,13 @@ pmem_map_fileU(const char *path, size_t len, int flags,
 	int fd;
 	int open_flags = O_RDWR;
 	int delete_on_err = 0;
-	int is_dev_dax = util_file_is_device_dax(path);
+	int file_type = util_file_get_type(path);
+#ifdef _WIN32
+	open_flags |= O_BINARY;
+#endif
+
+	if (file_type == OTHER_ERROR)
+		return NULL;
 
 	if (flags & ~(PMEM_FILE_ALL_FLAGS)) {
 		ERR("invalid flag specified %x", flags);
@@ -429,7 +436,7 @@ pmem_map_fileU(const char *path, size_t len, int flags,
 		return NULL;
 	}
 
-	if (is_dev_dax) {
+	if (file_type == TYPE_DEVDAX) {
 		if (flags & ~(PMEM_DAX_VALID_FLAGS)) {
 			ERR("flag unsupported for Device DAX %x", flags);
 			errno = EINVAL;
@@ -445,8 +452,8 @@ pmem_map_fileU(const char *path, size_t len, int flags,
 			}
 			if (len != 0 && len != (size_t)actual_len) {
 				ERR("Device DAX length must be either 0 or "
-					"the exact size of the device %zu",
-					len);
+					"the exact size of the device: %zu",
+					actual_len);
 				errno = EINVAL;
 				return NULL;
 			}
@@ -528,7 +535,7 @@ pmem_map_fileU(const char *path, size_t len, int flags,
 		len = (size_t)actual_size;
 	}
 
-	void *addr = pmem_map_register(fd, len, path, is_dev_dax);
+	void *addr = pmem_map_register(fd, len, path, file_type == TYPE_DEVDAX);
 	if (addr == NULL)
 		goto err;
 
@@ -599,6 +606,76 @@ pmem_unmap(void *addr, size_t len)
 	return util_unmap(addr, len);
 }
 
+/*
+ * pmem_memmove --  memmove to pmem
+ */
+void *
+pmem_memmove(void *pmemdest, const void *src, size_t len, unsigned flags)
+{
+	LOG(15, "pmemdest %p src %p len %zu flags 0x%x",
+			pmemdest, src, len, flags);
+
+#ifdef DEBUG
+	if (flags & ~PMEM_F_MEM_VALID_FLAGS)
+		ERR("invalid flags 0x%x", flags);
+#endif
+	PMEM_API_START();
+	Funcs.memmove_nodrain(pmemdest, src, len, flags & ~PMEM_F_MEM_NODRAIN);
+
+	if ((flags & (PMEM_F_MEM_NODRAIN | PMEM_F_MEM_NOFLUSH)) == 0)
+		pmem_drain();
+
+	PMEM_API_END();
+	return pmemdest;
+}
+
+/*
+ * pmem_memcpy --  memcpy to pmem
+ */
+void *
+pmem_memcpy(void *pmemdest, const void *src, size_t len, unsigned flags)
+{
+	LOG(15, "pmemdest %p src %p len %zu flags 0x%x",
+			pmemdest, src, len, flags);
+
+#ifdef DEBUG
+	if (flags & ~PMEM_F_MEM_VALID_FLAGS)
+		ERR("invalid flags 0x%x", flags);
+#endif
+	PMEM_API_START();
+	Funcs.memmove_nodrain(pmemdest, src, len, flags & ~PMEM_F_MEM_NODRAIN);
+
+	if ((flags & (PMEM_F_MEM_NODRAIN | PMEM_F_MEM_NOFLUSH)) == 0)
+		pmem_drain();
+
+	PMEM_API_END();
+	return pmemdest;
+}
+
+/*
+ * pmem_memset -- memset to pmem
+ */
+void *
+pmem_memset(void *pmemdest, int c, size_t len, unsigned flags)
+{
+	LOG(15, "pmemdest %p c 0x%x len %zu flags 0x%x",
+			pmemdest, c, len, flags);
+
+#ifdef DEBUG
+	if (flags & ~PMEM_F_MEM_VALID_FLAGS)
+		ERR("invalid flags 0x%x", flags);
+#endif
+
+	PMEM_API_START();
+	Funcs.memset_nodrain(pmemdest, c, len, flags & ~PMEM_F_MEM_NODRAIN);
+
+	if ((flags & (PMEM_F_MEM_NODRAIN | PMEM_F_MEM_NOFLUSH)) == 0)
+		pmem_drain();
+
+	PMEM_API_END();
+	return pmemdest;
+}
+
 /*
  * pmem_memmove_nodrain -- memmove to pmem without hw drain
  */
@@ -607,7 +684,12 @@ pmem_memmove_nodrain(void *pmemdest, const void *src, size_t len)
 {
 	LOG(15, "pmemdest %p src %p len %zu", pmemdest, src, len);
 
-	return Funcs.memmove_nodrain(pmemdest, src, len);
+	PMEM_API_START();
+
+	Funcs.memmove_nodrain(pmemdest, src, len, 0);
+
+	PMEM_API_END();
+	return pmemdest;
 }
 
 /*
@@ -618,7 +700,12 @@ pmem_memcpy_nodrain(void *pmemdest, const void *src, size_t len)
 {
 	LOG(15, "pmemdest %p src %p len %zu", pmemdest, src, len);
 
-	return pmem_memmove_nodrain(pmemdest, src, len);
+	PMEM_API_START();
+
+	Funcs.memmove_nodrain(pmemdest, src, len, 0);
+
+	PMEM_API_END();
+	return pmemdest;
 }
 
 /*
@@ -629,8 +716,12 @@ pmem_memmove_persist(void *pmemdest, const void *src, size_t len)
 {
 	LOG(15, "pmemdest %p src %p len %zu", pmemdest, src, len);
 
-	pmem_memmove_nodrain(pmemdest, src, len);
+	PMEM_API_START();
+
+	Funcs.memmove_nodrain(pmemdest, src, len, 0);
 	pmem_drain();
+
+	PMEM_API_END();
 	return pmemdest;
 }
 
@@ -642,8 +733,12 @@ pmem_memcpy_persist(void *pmemdest, const void *src, size_t len)
 {
 	LOG(15, "pmemdest %p src %p len %zu", pmemdest, src, len);
 
-	pmem_memcpy_nodrain(pmemdest, src, len);
+	PMEM_API_START();
+
+	Funcs.memmove_nodrain(pmemdest, src, len, 0);
 	pmem_drain();
+
+	PMEM_API_END();
 	return pmemdest;
 }
 
@@ -653,9 +748,14 @@ pmem_memcpy_persist(void *pmemdest, const void *src, size_t len)
 void *
 pmem_memset_nodrain(void *pmemdest, int c, size_t len)
 {
-	LOG(15, "pmemdest %p c 0x%x len %zu", pmemdest, c, len);
+	LOG(15, "pmemdest %p c %d len %zu", pmemdest, c, len);
 
-	return Funcs.memset_nodrain(pmemdest, c, len);
+	PMEM_API_START();
+
+	Funcs.memset_nodrain(pmemdest, c, len, 0);
+
+	PMEM_API_END();
+	return pmemdest;
 }
 
 /*
@@ -664,10 +764,14 @@ pmem_memset_nodrain(void *pmemdest, int c, size_t len)
 void *
 pmem_memset_persist(void *pmemdest, int c, size_t len)
 {
-	LOG(15, "pmemdest %p c 0x%x len %zu", pmemdest, c, len);
+	LOG(15, "pmemdest %p c %d len %zu", pmemdest, c, len);
 
-	pmem_memset_nodrain(pmemdest, c, len);
+	PMEM_API_START();
+
+	Funcs.memset_nodrain(pmemdest, c, len, 0);
 	pmem_drain();
+
+	PMEM_API_END();
 	return pmemdest;
 }
 
@@ -708,3 +812,29 @@ pmem_deep_drain(const void *addr, size_t len)
 
 	return os_range_deep_common((uintptr_t)addr, len);
 }
+
+#if VG_PMEMCHECK_ENABLED
+/*
+ * pmem_emit_log -- logs library and function names to pmemcheck store log
+ */
+void
+pmem_emit_log(const char *func, int order)
+{
+	util_emit_log("libpmem", func, order);
+}
+#endif
+
+#if FAULT_INJECTION
+void
+pmem_inject_fault_at(enum pmem_allocation_type type, int nth,
+						const char *at)
+{
+	common_inject_fault_at(type, nth, at);
+}
+
+int
+pmem_fault_injection_enabled(void)
+{
+	return common_fault_injection_enabled();
+}
+#endif
