@@ -1,5 +1,5 @@
 /*
- * Copyright 2018, Intel Corporation
+ * Copyright 2018-2020, Intel Corporation
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -32,7 +32,7 @@
 
 /*
  * ravl.c -- implementation of a RAVL tree
- * http://sidsen.azurewebsites.net//papers/ravl-trees-journal.pdf
+ * https://sidsen.azurewebsites.net//papers/ravl-trees-journal.pdf
  */
 
 #include <stdlib.h>
@@ -40,6 +40,7 @@
 #include <errno.h>
 #include "out.h"
 #include "ravl.h"
+#include "alloc.h"
 
 #define RAVL_DEFAULT_DATA_SIZE (sizeof(void *))
 
@@ -73,8 +74,10 @@ struct ravl *
 ravl_new_sized(ravl_compare *compare, size_t data_size)
 {
 	struct ravl *r = Malloc(sizeof(*r));
-	if (r == NULL)
-		return NULL;
+	if (r == NULL) {
+		ERR("!Malloc");
+		return r;
+	}
 
 	r->compare = compare;
 	r->root = NULL;
@@ -94,20 +97,21 @@ ravl_new(ravl_compare *compare)
 
 /*
  * ravl_clear_node -- (internal) recursively clears the given subtree,
- *	calls callback in an in-order fashion. Frees the given node.
+ *	calls callback in an in-order fashion. Optionally frees the given node.
  */
 static void
-ravl_clear_node(struct ravl_node *n, ravl_cb cb, void *arg)
+ravl_foreach_node(struct ravl_node *n, ravl_cb cb, void *arg, int free_node)
 {
 	if (n == NULL)
 		return;
 
-	ravl_clear_node(n->slots[RAVL_LEFT], cb, arg);
+	ravl_foreach_node(n->slots[RAVL_LEFT], cb, arg, free_node);
 	if (cb)
 		cb((void *)n->data, arg);
-	ravl_clear_node(n->slots[RAVL_RIGHT], cb, arg);
+	ravl_foreach_node(n->slots[RAVL_RIGHT], cb, arg, free_node);
 
-	Free(n);
+	if (free_node)
+		Free(n);
 }
 
 /*
@@ -116,7 +120,7 @@ ravl_clear_node(struct ravl_node *n, ravl_cb cb, void *arg)
 void
 ravl_clear(struct ravl *ravl)
 {
-	ravl_clear_node(ravl->root, NULL, NULL);
+	ravl_foreach_node(ravl->root, NULL, NULL, 1);
 	ravl->root = NULL;
 }
 
@@ -126,7 +130,7 @@ ravl_clear(struct ravl *ravl)
 void
 ravl_delete_cb(struct ravl *ravl, ravl_cb cb, void *arg)
 {
-	ravl_clear_node(ravl->root, cb, arg);
+	ravl_foreach_node(ravl->root, cb, arg, 1);
 	Free(ravl);
 }
 
@@ -139,6 +143,15 @@ ravl_delete(struct ravl *ravl)
 	ravl_delete_cb(ravl, NULL, NULL);
 }
 
+/*
+ * ravl_foreach -- traverses the entire tree, calling callback for every node
+ */
+void
+ravl_foreach(struct ravl *ravl, ravl_cb cb, void *arg)
+{
+	ravl_foreach_node(ravl->root, cb, arg, 0);
+}
+
 /*
  * ravl_empty -- checks whether the given tree is empty
  */
@@ -174,8 +187,10 @@ static struct ravl_node *
 ravl_new_node(struct ravl *ravl, ravl_constr constr, const void *arg)
 {
 	struct ravl_node *n = Malloc(sizeof(*n) + ravl->data_size);
-	if (n == NULL)
-		return NULL;
+	if (n == NULL) {
+		ERR("!Malloc");
+		return n;
+	}
 
 	n->parent = NULL;
 	n->slots[RAVL_LEFT] = NULL;
