@@ -1,5 +1,5 @@
 /*
- * Copyright 2016-2018, Intel Corporation
+ * Copyright 2016-2019, Intel Corporation
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -52,6 +52,9 @@
 extern "C" {
 #endif
 
+#include "alloc.h"
+#include "fault_injection.h"
+
 enum pool_type {
 	POOL_TYPE_UNKNOWN	= (1 << 0),
 	POOL_TYPE_LOG		= (1 << 1),
@@ -94,7 +97,7 @@ struct pool_set_file {
 };
 
 struct arena {
-	TAILQ_ENTRY(arena) next;
+	PMDK_TAILQ_ENTRY(arena) next;
 	struct btt_info btt_info;
 	uint32_t id;
 	bool valid;
@@ -121,7 +124,7 @@ struct pool_data {
 		UUID_NOT_FROM_BTT,
 	} uuid_op;
 	struct arena bttc;
-	TAILQ_HEAD(arenashead, arena) arenas;
+	PMDK_TAILQ_HEAD(arenashead, arena) arenas;
 	uint32_t narenas;
 };
 
@@ -160,6 +163,28 @@ uint64_t pool_get_first_valid_btt(struct pool_data *pool,
 	struct btt_info *infop, uint64_t offset, bool *zeroed);
 size_t pool_get_min_size(enum pool_type);
 
+#if FAULT_INJECTION
+void
+pmempool_inject_fault_at(enum pmem_allocation_type type, int nth,
+							const char *at);
+
+int
+pmempool_fault_injection_enabled(void);
+#else
+static inline void
+pmempool_inject_fault_at(enum pmem_allocation_type type, int nth,
+						const char *at)
+{
+	abort();
+}
+
+static inline int
+pmempool_fault_injection_enabled(void)
+{
+	return 0;
+}
+#endif
+
 #ifdef __cplusplus
 }
 #endif
