@@ -1,5 +1,5 @@
 /*
- * Copyright 2015-2018, Intel Corporation
+ * Copyright 2015-2019, Intel Corporation
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -54,7 +54,7 @@ TOID_DECLARE(struct object, 0);
 
 struct object {
 	size_t value;
-	char data[OBJ_SIZE - sizeof(size_t)];
+	unsigned char data[OBJ_SIZE - sizeof(size_t)];
 };
 
 #define VALUE_OFF	(offsetof(struct object, value))
@@ -65,10 +65,10 @@ struct object {
 #define TEST_VALUE_2	2
 
 /*
- * do_tx_alloc -- do tx allocation with specified type number
+ * do_tx_zalloc -- do tx allocation with specified type number
  */
 static PMEMoid
-do_tx_zalloc(PMEMobjpool *pop, int type_num)
+do_tx_zalloc(PMEMobjpool *pop, unsigned type_num)
 {
 	PMEMoid ret = OID_NULL;
 
@@ -79,6 +79,22 @@ do_tx_zalloc(PMEMobjpool *pop, int type_num)
 	return ret;
 }
 
+/*
+ * do_tx_alloc -- do tx allocation and initialize first num bytes
+ */
+static PMEMoid
+do_tx_alloc(PMEMobjpool *pop, uint64_t type_num, uint64_t init_num)
+{
+	PMEMoid ret = OID_NULL;
+
+	TX_BEGIN(pop) {
+		ret = pmemobj_tx_alloc(sizeof(struct object), type_num);
+		pmemobj_memset(pop, pmemobj_direct(ret), 0, init_num, 0);
+	} TX_END
+
+	return ret;
+}
+
 /*
  * do_tx_add_range_alloc_commit -- call add_range_direct on object allocated
  * within the same transaction and commit the transaction
@@ -403,10 +419,11 @@ do_tx_add_range_commit(PMEMobjpool *pop)
 }
 
 /*
- * do_tx_xadd_range_commit -- call xadd_range_direct and commit tx
+ * do_tx_xadd_range_no_flush_commit -- call xadd_range_direct with
+ * POBJ_XADD_NO_FLUSH flag set and commit tx
  */
 static void
-do_tx_xadd_range_commit(PMEMobjpool *pop)
+do_tx_xadd_range_no_flush_commit(PMEMobjpool *pop)
 {
 	int ret;
 	TOID(struct object) obj;
@@ -427,6 +444,208 @@ do_tx_xadd_range_commit(PMEMobjpool *pop)
 	UT_ASSERTeq(D_RO(obj)->value, TEST_VALUE_1);
 }
 
+/*
+ * do_tx_xadd_range_no_snapshot_commit -- call xadd_range_direct with
+ * POBJ_XADD_NO_SNAPSHOT flag, commit the transaction
+ */
+static void
+do_tx_xadd_range_no_snapshot_commit(PMEMobjpool *pop)
+{
+	int ret;
+	TOID(struct object) obj;
+	TOID_ASSIGN(obj, do_tx_zalloc(pop, TYPE_OBJ));
+
+	TX_BEGIN(pop) {
+		char *ptr = (char *)pmemobj_direct(obj.oid);
+		ret = pmemobj_tx_xadd_range_direct(ptr + VALUE_OFF,
+				VALUE_SIZE, POBJ_XADD_NO_SNAPSHOT);
+		UT_ASSERTeq(ret, 0);
+
+		D_RW(obj)->value = TEST_VALUE_1;
+	} TX_ONABORT {
+		UT_ASSERT(0);
+	} TX_END
+
+	UT_ASSERTeq(D_RO(obj)->value, TEST_VALUE_1);
+}
+
+/*
+ * do_tx_xadd_range_no_snapshot_abort -- call xadd_range_direct with
+ * POBJ_XADD_NO_SNAPSHOT flag, modify the value, abort the transaction
+ */
+static void
+do_tx_xadd_range_no_snapshot_abort(PMEMobjpool *pop)
+{
+	int ret;
+	TOID(struct object) obj;
+	TOID_ASSIGN(obj, do_tx_zalloc(pop, TYPE_OBJ));
+	D_RW(obj)->value = TEST_VALUE_1;
+
+	TX_BEGIN(pop) {
+		char *ptr = (char *)pmemobj_direct(obj.oid);
+		ret = pmemobj_tx_xadd_range_direct(ptr + VALUE_OFF, VALUE_SIZE,
+				POBJ_XADD_NO_SNAPSHOT);
+		UT_ASSERTeq(ret, 0);
+		D_RW(obj)->value = TEST_VALUE_2;
+		pmemobj_tx_abort(-1);
+	} TX_ONCOMMIT {
+		UT_ASSERT(0);
+	} TX_END
+
+	/*
+	 * value added with NO_SNAPSHOT flag should NOT be rolled back
+	 * after abort
+	 */
+	UT_ASSERTeq(D_RO(obj)->value, TEST_VALUE_2);
+}
+
+/*
+ * do_tx_xadd_range_no_uninit_check -- call xdd_range_direct for
+ * initialized memory with POBJ_XADD_ASSUME_INITIALIZED flag set and commit the
+ * tx
+ */
+static void
+do_tx_xadd_range_no_uninit_check_commit(PMEMobjpool *pop)
+{
+	int ret;
+	TOID(struct object) obj;
+	TOID_ASSIGN(obj, do_tx_zalloc(pop, TYPE_OBJ));
+
+	TX_BEGIN(pop) {
+		char *ptr = (char *)pmemobj_direct(obj.oid);
+		ret = pmemobj_tx_xadd_range_direct(ptr + VALUE_OFF, VALUE_SIZE,
+				POBJ_XADD_ASSUME_INITIALIZED);
+		UT_ASSERTeq(ret, 0);
+		D_RW(obj)->value = TEST_VALUE_1;
+	} TX_ONABORT {
+		UT_ASSERT(0);
+	} TX_END
+
+	UT_ASSERTeq(D_RO(obj)->value, TEST_VALUE_1);
+}
+
+/*
+ * do_tx_xadd_range_no_uninit_check -- call xadd_range_direct for
+ * uninitialized memory with POBJ_XADD_ASSUME_INITIALIZED flag set and commit
+ * the tx
+ */
+static void
+do_tx_xadd_range_no_uninit_check_commit_uninit(PMEMobjpool *pop)
+{
+	int ret;
+	TOID(struct object) obj;
+	TOID_ASSIGN(obj, do_tx_alloc(pop, TYPE_OBJ, 0));
+
+	TX_BEGIN(pop) {
+		char *ptr = (char *)pmemobj_direct(obj.oid);
+		ret = pmemobj_tx_xadd_range_direct(ptr + VALUE_OFF, VALUE_SIZE,
+				POBJ_XADD_ASSUME_INITIALIZED);
+		UT_ASSERTeq(ret, 0);
+
+		ret = pmemobj_tx_xadd_range_direct(ptr + DATA_OFF, DATA_SIZE,
+				POBJ_XADD_ASSUME_INITIALIZED);
+		UT_ASSERTeq(ret, 0);
+
+		D_RW(obj)->value = TEST_VALUE_1;
+		D_RW(obj)->data[256] = TEST_VALUE_2;
+	} TX_ONABORT {
+		UT_ASSERT(0);
+	} TX_END
+
+	UT_ASSERTeq(D_RO(obj)->value, TEST_VALUE_1);
+	UT_ASSERTeq(D_RO(obj)->data[256], TEST_VALUE_2);
+}
+
+/*
+ * do_tx_xadd_range_no_uninit_check -- call xadd_range_direct for
+ * partially uninitialized memory with POBJ_XADD_ASSUME_INITIALIZED flag set
+ * only for uninitialized part and commit the tx
+ */
+static void
+do_tx_xadd_range_no_uninit_check_commit_part_uninit(PMEMobjpool *pop)
+{
+	int ret;
+	TOID(struct object) obj;
+	TOID_ASSIGN(obj, do_tx_alloc(pop, TYPE_OBJ, VALUE_SIZE));
+
+	TX_BEGIN(pop) {
+		char *ptr = (char *)pmemobj_direct(obj.oid);
+		ret = pmemobj_tx_add_range_direct(ptr + VALUE_OFF, VALUE_SIZE);
+		UT_ASSERTeq(ret, 0);
+
+		ret = pmemobj_tx_xadd_range_direct(ptr + DATA_OFF, DATA_SIZE,
+				POBJ_XADD_ASSUME_INITIALIZED);
+		UT_ASSERTeq(ret, 0);
+
+		D_RW(obj)->value = TEST_VALUE_1;
+		D_RW(obj)->data[256] = TEST_VALUE_2;
+	} TX_ONABORT {
+		UT_ASSERT(0);
+	} TX_END
+
+	UT_ASSERTeq(D_RO(obj)->value, TEST_VALUE_1);
+	UT_ASSERTeq(D_RO(obj)->data[256], TEST_VALUE_2);
+}
+
+/*
+ * do_tx_add_range_no_uninit_check -- call add_range_direct for
+ * partially uninitialized memory.
+ */
+static void
+do_tx_add_range_no_uninit_check_commit_no_flag(PMEMobjpool *pop)
+{
+	int ret;
+	TOID(struct object) obj;
+	TOID_ASSIGN(obj, do_tx_alloc(pop, TYPE_OBJ, VALUE_SIZE));
+
+	TX_BEGIN(pop) {
+		char *ptr = (char *)pmemobj_direct(obj.oid);
+		ret = pmemobj_tx_add_range_direct(ptr + VALUE_OFF, VALUE_SIZE);
+		UT_ASSERTeq(ret, 0);
+
+		ret = pmemobj_tx_add_range_direct(ptr + DATA_OFF, DATA_SIZE);
+		UT_ASSERTeq(ret, 0);
+
+		D_RW(obj)->value = TEST_VALUE_1;
+		D_RW(obj)->data[256] = TEST_VALUE_2;
+	} TX_ONABORT {
+		UT_ASSERT(0);
+	} TX_END
+
+	UT_ASSERTeq(D_RO(obj)->value, TEST_VALUE_1);
+	UT_ASSERTeq(D_RO(obj)->data[256], TEST_VALUE_2);
+}
+
+/*
+ * do_tx_xadd_range_no_uninit_check_abort -- call pmemobj_tx_range with
+ * POBJ_XADD_ASSUME_INITIALIZED flag, modify the value inside aborted
+ * transaction
+ */
+static void
+do_tx_xadd_range_no_uninit_check_abort(PMEMobjpool *pop)
+{
+	int ret;
+	TOID(struct object) obj;
+	TOID_ASSIGN(obj, do_tx_alloc(pop, TYPE_OBJ, 0));
+
+	TX_BEGIN(pop) {
+		char *ptr = (char *)pmemobj_direct(obj.oid);
+		ret = pmemobj_tx_xadd_range_direct(ptr + VALUE_OFF, VALUE_SIZE,
+				POBJ_XADD_ASSUME_INITIALIZED);
+		UT_ASSERTeq(ret, 0);
+
+		ret = pmemobj_tx_xadd_range_direct(ptr + DATA_OFF, DATA_SIZE,
+				POBJ_XADD_ASSUME_INITIALIZED);
+		UT_ASSERTeq(ret, 0);
+
+		D_RW(obj)->value = TEST_VALUE_1;
+		D_RW(obj)->data[256] = TEST_VALUE_2;
+		pmemobj_tx_abort(-1);
+	} TX_ONCOMMIT {
+		UT_ASSERT(0);
+	} TX_END
+}
+
 /*
  * do_tx_commit_and_abort -- use range cache, commit and then abort to make
  *	sure that it won't affect previously modified data.
@@ -540,15 +759,30 @@ do_tx_add_range_too_large(PMEMobjpool *pop)
 {
 	TOID(struct object) obj;
 	TOID_ASSIGN(obj, do_tx_zalloc(pop, TYPE_OBJ));
-
+	int ret = 0;
 	TX_BEGIN(pop) {
-		pmemobj_tx_add_range_direct(pmemobj_direct(obj.oid),
+		ret = pmemobj_tx_add_range_direct(pmemobj_direct(obj.oid),
 			PMEMOBJ_MAX_ALLOC_SIZE + 1);
 	} TX_ONCOMMIT {
 		UT_ASSERT(0);
+	} TX_ONABORT {
+		UT_ASSERTeq(errno, EINVAL);
+		UT_ASSERTeq(ret, 0);
+	} TX_END
+
+	errno = 0;
+	ret = 0;
+
+	TX_BEGIN(pop) {
+		ret = pmemobj_tx_xadd_range_direct(pmemobj_direct(obj.oid),
+				PMEMOBJ_MAX_ALLOC_SIZE + 1, POBJ_XADD_NO_ABORT);
+	} TX_ONCOMMIT {
+		UT_ASSERTeq(errno, EINVAL);
+		UT_ASSERTeq(ret, EINVAL);
+	} TX_ONABORT {
+		UT_ASSERT(0);
 	} TX_END
 
-	UT_ASSERTne(errno, 0);
 	errno = 0;
 }
 
@@ -569,8 +803,6 @@ do_tx_add_range_lots_of_small_snapshots(PMEMobjpool *pop)
 	} TX_ONABORT {
 		UT_ASSERT(0);
 	} TX_END
-
-	UT_ASSERTeq(errno, 0);
 }
 
 static void
@@ -652,8 +884,21 @@ main(int argc, char *argv[])
 	VALGRIND_WRITE_STATS;
 	do_tx_add_cache_overflowing_range(pop);
 	VALGRIND_WRITE_STATS;
-	do_tx_xadd_range_commit(pop);
-
+	do_tx_xadd_range_no_snapshot_commit(pop);
+	VALGRIND_WRITE_STATS;
+	do_tx_xadd_range_no_snapshot_abort(pop);
+	VALGRIND_WRITE_STATS;
+	do_tx_xadd_range_no_uninit_check_commit(pop);
+	VALGRIND_WRITE_STATS;
+	do_tx_xadd_range_no_uninit_check_commit_uninit(pop);
+	VALGRIND_WRITE_STATS;
+	do_tx_xadd_range_no_uninit_check_commit_part_uninit(pop);
+	VALGRIND_WRITE_STATS;
+	do_tx_xadd_range_no_uninit_check_abort(pop);
+	VALGRIND_WRITE_STATS;
+	do_tx_add_range_no_uninit_check_commit_no_flag(pop);
+	VALGRIND_WRITE_STATS;
+	do_tx_xadd_range_no_flush_commit(pop);
 	pmemobj_close(pop);
 
 	DONE(NULL);
