@@ -1,5 +1,5 @@
 /*
- * Copyright 2016-2018, Intel Corporation
+ * Copyright 2016-2020, Intel Corporation
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -38,6 +38,7 @@
 
 #include "librpmem.h"
 #include "pool_hdr.h"
+#include "file.h"
 #include "set.h"
 #include "util.h"
 #include "out.h"
@@ -53,7 +54,7 @@
 	.signature		= "<RPMEM>",\
 	.major			= 1,\
 	.compat_features	= 2,\
-	.incompat_features	= 0,\
+	.incompat_features	= 2,\
 	.ro_compat_features	= 4,\
 	.poolset_uuid		= "POOLSET_UUID0123",\
 	.uuid			= "UUID0123456789AB",\
@@ -67,7 +68,7 @@
 	.signature		= "<RPMEM>",\
 	.major			= 1,\
 	.compat_features	= 2,\
-	.incompat_features	= 1,\
+	.incompat_features	= 3,\
 	.ro_compat_features	= 4,\
 	.poolset_uuid		= "POOLSET_UUID0123",\
 	.uuid			= "UUID0123456789AB",\
@@ -80,7 +81,7 @@
 	.signature		= "<ALT>",\
 	.major			= 5,\
 	.compat_features	= 6,\
-	.incompat_features	= 0,\
+	.incompat_features	= 2,\
 	.ro_compat_features	= 8,\
 	.poolset_uuid		= "UUID_POOLSET_ALT",\
 	.uuid			= "ALT_UUIDCDEFFEDC",\
@@ -94,7 +95,7 @@
 	.signature		= "<ALT>",\
 	.major			= 5,\
 	.compat_features	= 6,\
-	.incompat_features	= 1,\
+	.incompat_features	= 3,\
 	.ro_compat_features	= 8,\
 	.poolset_uuid		= "UUID_POOLSET_ALT",\
 	.uuid			= "ALT_UUIDCDEFFEDC",\
@@ -103,14 +104,14 @@
 	.user_flags		= "\0\0\0\n~._ALT_FLAGS",\
 }
 
-const struct rpmem_pool_attr pool_attrs[] = {
+static const struct rpmem_pool_attr pool_attrs[] = {
 	POOL_ATTR_INIT,
 	POOL_ATTR_INIT_SINGLEHDR,
 	POOL_ATTR_ALT,
 	POOL_ATTR_ALT_SINGLEHDR
 };
 
-const char *pool_attr_names[] = {
+static const char *pool_attr_names[] = {
 	"init",
 	"init_singlehdr",
 	"alt",
@@ -126,6 +127,8 @@ struct pool_entry {
 	const char *target;
 	void *pool;
 	size_t size;
+	size_t buff_offset; /* working buffer offset */
+	size_t buff_size; /* working buffer size */
 	unsigned nlanes;
 	int is_mem;
 	int error_must_occur;
@@ -133,7 +136,17 @@ struct pool_entry {
 };
 
 #define MAX_IDS	1024
-struct pool_entry pools[MAX_IDS];
+static struct pool_entry pools[MAX_IDS];
+
+/*
+ * init_buff -- default working buffer parameters
+ */
+static inline void
+init_buff(struct pool_entry *pool)
+{
+	pool->buff_offset = POOL_HDR_SIZE;
+	pool->buff_size = pool->size - POOL_HDR_SIZE;
+}
 
 /*
  * init_pool -- map local pool file or allocate memory region
@@ -153,7 +166,6 @@ init_pool(struct pool_entry *pool, const char *target, const char *pool_path,
 	if (pool->size)
 		flags |= PMEM_FILE_EXCL;
 
-
 	if (strncmp(pool_path, "mem", strlen("mem")) == 0) {
 		pool->pool = PAGEALIGNMALLOC(pool->size);
 
@@ -183,8 +195,12 @@ init_pool(struct pool_entry *pool, const char *target, const char *pool_path,
 		UT_ASSERTeq(ret, 0);
 
 		pool->is_mem = 0;
-		os_unlink(pool_path);
+
+		if (util_file_get_type(pool_path) != TYPE_DEVDAX)
+			os_unlink(pool_path);
 	}
+
+	init_buff(pool);
 }
 
 /*
@@ -255,12 +271,12 @@ cmp_pool_attr(const struct rpmem_pool_attr *attr1,
  * check_return_and_errno - validate return value and errno
  */
 #define check_return_and_errno(ret, error_must_occur, exp_errno) \
-	if (exp_errno != 0) { \
+	if ((exp_errno) != 0) { \
 		if (error_must_occur) { \
 			UT_ASSERTne(ret, 0); \
 			UT_ASSERTeq(errno, exp_errno); \
 		} else { \
-			if (ret != 0) { \
+			if ((ret) != 0) { \
 				UT_ASSERTeq(errno, exp_errno); \
 			} \
 		} \
@@ -280,8 +296,8 @@ static int
 test_create(const struct test_case *tc, int argc, char *argv[])
 {
 	if (argc < 6)
-		UT_FATAL("usage: test_create <id> <pool set> "
-				"<target> <pool> <size> <option>");
+		UT_FATAL(
+			"usage: test_create <id> <pool set> <target> <pool> <size> <option>");
 
 	const char *id_str = argv[0];
 	const char *pool_set = argv[1];
@@ -334,9 +350,8 @@ static int
 test_open(const struct test_case *tc, int argc, char *argv[])
 {
 	if (argc < 7)
-		UT_FATAL("usage: test_open <id> <pool set> "
-				"<target> <pool> <size> <pool attr name> "
-				"<option>");
+		UT_FATAL(
+			"usage: test_open <id> <pool set> <target> <pool> <size> <pool attr name> <option>");
 
 	const char *id_str = argv[0];
 	const char *pool_set = argv[1];
@@ -405,17 +420,41 @@ test_close(const struct test_case *tc, int argc, char *argv[])
 	return 1;
 }
 
-typedef int (*flush_func)(RPMEMpool *rpp, size_t off,
-		size_t size, unsigned lane);
+typedef int (*flush_func)(
+		RPMEMpool *rpp, size_t off, size_t size, unsigned lane);
+
+static int
+persist_relaxed(RPMEMpool *rpp, size_t off, size_t size, unsigned lane)
+{
+	return rpmem_persist(rpp, off, size, lane, RPMEM_PERSIST_RELAXED);
+}
+
+static int
+persist_normal(RPMEMpool *rpp, size_t off, size_t size, unsigned lane)
+{
+	return rpmem_persist(rpp, off, size, lane, 0);
+}
+
+static int
+flush_relaxed(RPMEMpool *rpp, size_t off, size_t size, unsigned lane)
+{
+	return rpmem_flush(rpp, off, size, lane, RPMEM_FLUSH_RELAXED);
+}
+
+static int
+flush_normal(RPMEMpool *rpp, size_t off, size_t size, unsigned lane)
+{
+	return rpmem_flush(rpp, off, size, lane, 0);
+}
 
 /*
- * thread_arg -- persist worker thread arguments
+ * flush_thread_arg -- flush worker thread arguments
  */
-struct thread_arg {
+struct flush_thread_arg {
 	RPMEMpool *rpp;
 	size_t off;
 	size_t size;
-	int nops;
+	unsigned nops;
 	unsigned lane;
 	int error_must_occur;
 	int exp_errno;
@@ -424,15 +463,15 @@ struct thread_arg {
 };
 
 /*
- * thread_func -- worker thread function
+ * flush_thread_func -- worker thread function for flushing ops
  */
 static void *
-thread_func(void *arg)
+flush_thread_func(void *arg)
 {
-	struct thread_arg *args = arg;
+	struct flush_thread_arg *args = arg;
 	size_t flush_size = args->size / args->nops;
 	UT_ASSERTeq(args->size % args->nops, 0);
-	for (int i = 0; i < args->nops; i++) {
+	for (unsigned i = 0; i < args->nops; i++) {
 		size_t off = args->off + i * flush_size;
 		size_t left = args->size - i * flush_size;
 		size_t size = left < flush_size ?
@@ -447,45 +486,44 @@ thread_func(void *arg)
 }
 
 static void
-test_flush(int id, int seed, int nthreads, int nops, flush_func func)
+test_flush_imp(unsigned id, unsigned seed, unsigned nthreads, unsigned nops,
+	flush_func func)
 {
 	struct pool_entry *pool = &pools[id];
 
 	UT_ASSERTne(pool->nlanes, 0);
 	nthreads = min(nthreads, pool->nlanes);
 
-	size_t buff_size = pool->size - POOL_HDR_SIZE;
-
 	if (seed) {
 		srand(seed);
 		uint8_t *buff = (uint8_t *)((uintptr_t)pool->pool +
-				POOL_HDR_SIZE);
-		for (size_t i = 0; i < buff_size; i++)
+				pool->buff_offset);
+		for (size_t i = 0; i < pool->buff_size; i++)
 			buff[i] = rand();
 	}
 
 	os_thread_t *threads = MALLOC(nthreads * sizeof(*threads));
-	struct thread_arg *args = MALLOC(nthreads * sizeof(*args));
-	size_t size_per_thread = buff_size / nthreads;
-	UT_ASSERTeq(buff_size % nthreads, 0);
+	struct flush_thread_arg *args = MALLOC(nthreads * sizeof(*args));
+	size_t size_per_thread = pool->buff_size / nthreads;
+	UT_ASSERTeq(pool->buff_size % nthreads, 0);
 
-	for (int i = 0; i < nthreads; i++) {
+	for (unsigned i = 0; i < nthreads; i++) {
 		args[i].rpp = pool->rpp;
 		args[i].nops = nops;
 		args[i].lane = (unsigned)i;
-		args[i].off = POOL_HDR_SIZE + i * size_per_thread;
+		args[i].off = pool->buff_offset + i * size_per_thread;
 		args[i].flush = func;
-		size_t size_left = buff_size - size_per_thread * i;
+		size_t size_left = pool->buff_size - size_per_thread * i;
 		args[i].size = size_left < size_per_thread ?
 				size_left : size_per_thread;
 		args[i].exp_errno = pool->exp_errno;
 		args[i].error_must_occur = pool->error_must_occur;
 
-		PTHREAD_CREATE(&threads[i], NULL, thread_func, &args[i]);
+		THREAD_CREATE(&threads[i], NULL, flush_thread_func, &args[i]);
 	}
 
 	for (int i = 0; i < nthreads; i++)
-		PTHREAD_JOIN(&threads[i], NULL);
+		THREAD_JOIN(&threads[i], NULL);
 
 	FREE(args);
 	FREE(threads);
@@ -497,18 +535,23 @@ test_flush(int id, int seed, int nthreads, int nops, flush_func func)
 static int
 test_persist(const struct test_case *tc, int argc, char *argv[])
 {
-	if (argc < 4)
-		UT_FATAL("usage: test_persist <id> <seed> <nthreads> <nops>");
+	if (argc < 5)
+		UT_FATAL(
+			"usage: test_persist <id> <seed> <nthreads> <nops> <relaxed>");
 
-	int id = atoi(argv[0]);
+	unsigned id = ATOU(argv[0]);
 	UT_ASSERT(id >= 0 && id < MAX_IDS);
-	int seed = atoi(argv[1]);
-	int nthreads = atoi(argv[2]);
-	int nops = atoi(argv[3]);
+	unsigned seed = ATOU(argv[1]);
+	unsigned nthreads = ATOU(argv[2]);
+	unsigned nops = ATOU(argv[3]);
+	unsigned relaxed = ATOU(argv[4]);
 
-	test_flush(id, seed, nthreads, nops, rpmem_persist);
+	if (relaxed)
+		test_flush_imp(id, seed, nthreads, nops, persist_relaxed);
+	else
+		test_flush_imp(id, seed, nthreads, nops, persist_normal);
 
-	return 4;
+	return 5;
 }
 
 /*
@@ -518,20 +561,118 @@ static int
 test_deep_persist(const struct test_case *tc, int argc, char *argv[])
 {
 	if (argc < 4)
-		UT_FATAL("usage: test_deep_persist <id> <seed> <nthreads> "
-				"<nops>");
+		UT_FATAL(
+			"usage: test_deep_persist <id> <seed> <nthreads> <nops>");
 
-	int id = atoi(argv[0]);
+	unsigned id = ATOU(argv[0]);
 	UT_ASSERT(id >= 0 && id < MAX_IDS);
-	int seed = atoi(argv[1]);
-	int nthreads = atoi(argv[2]);
-	int nops = atoi(argv[3]);
+	unsigned seed = ATOU(argv[1]);
+	unsigned nthreads = ATOU(argv[2]);
+	unsigned nops = ATOU(argv[3]);
 
-	test_flush(id, seed, nthreads, nops, rpmem_deep_persist);
+	test_flush_imp(id, seed, nthreads, nops, rpmem_deep_persist);
 
 	return 4;
 }
 
+/*
+ * test_flush -- test case for flush operation
+ */
+static int
+test_flush(const struct test_case *tc, int argc, char *argv[])
+{
+	if (argc < 4)
+		UT_FATAL("usage: test_flush <id> <seed> <nthreads> <nops> "
+				"<relaxed>");
+
+	unsigned id = ATOU(argv[0]);
+	UT_ASSERT(id >= 0 && id < MAX_IDS);
+	unsigned seed = ATOU(argv[1]);
+	unsigned nthreads = ATOU(argv[2]);
+	unsigned nops = ATOU(argv[3]);
+	unsigned relaxed = ATOU(argv[4]);
+
+	if (relaxed)
+		test_flush_imp(id, seed, nthreads, nops, flush_relaxed);
+	else
+		test_flush_imp(id, seed, nthreads, nops, flush_normal);
+
+	return 5;
+}
+
+/*
+ * drain_thread_arg -- drain worker thread arguments
+ */
+struct drain_thread_arg {
+	RPMEMpool *rpp;
+	unsigned nops;
+	unsigned lane;
+	int error_must_occur;
+	int exp_errno;
+};
+
+/*
+ * drain_thread_func -- worker thread function for drain
+ */
+static void *
+drain_thread_func(void *arg)
+{
+	struct drain_thread_arg *args = arg;
+	UT_ASSERTeq(args->nops, 1);
+
+	int ret = rpmem_drain(args->rpp, args->lane, 0 /* flags */);
+	check_return_and_errno(ret, args->error_must_occur,
+		args->exp_errno);
+
+	return NULL;
+}
+
+static void
+test_drain_imp(unsigned id, unsigned nthreads)
+{
+	struct pool_entry *pool = &pools[id];
+
+	UT_ASSERTne(pool->nlanes, 0);
+	nthreads = min(nthreads, pool->nlanes);
+
+	os_thread_t *threads = MALLOC(nthreads * sizeof(*threads));
+	struct drain_thread_arg *args = MALLOC(nthreads * sizeof(*args));
+
+	for (unsigned i = 0; i < nthreads; i++) {
+		args[i].rpp = pool->rpp;
+		args[i].nops = 1;
+		args[i].lane = (unsigned)i;
+		args[i].exp_errno = pool->exp_errno;
+		args[i].error_must_occur = pool->error_must_occur;
+
+		THREAD_CREATE(&threads[i], NULL, drain_thread_func, &args[i]);
+	}
+
+	for (int i = 0; i < nthreads; i++)
+		THREAD_JOIN(&threads[i], NULL);
+
+	FREE(args);
+	FREE(threads);
+}
+
+/*
+ * test_drain -- test case for drain operation
+ */
+static int
+test_drain(const struct test_case *tc, int argc, char *argv[])
+{
+	if (argc < 2)
+		UT_FATAL("usage: test_drain <id> <nthreads>");
+
+	unsigned id = ATOU(argv[0]);
+	UT_ASSERT(id >= 0 && id < MAX_IDS);
+	unsigned nthreads = ATOU(argv[1]);
+
+	test_drain_imp(id, nthreads);
+
+	return 2;
+}
+
 /*
  * test_read -- test case for read operation
  */
@@ -545,7 +686,7 @@ test_read(const struct test_case *tc, int argc, char *argv[])
 	UT_ASSERT(id >= 0 && id < MAX_IDS);
 	struct pool_entry *pool = &pools[id];
 
-	srand(atoi(argv[1]));
+	srand(ATOU(argv[1]));
 
 	int ret;
 	uint8_t *buff = (uint8_t *)((uintptr_t)pool->pool + POOL_HDR_SIZE);
@@ -571,8 +712,8 @@ static int
 test_remove(const struct test_case *tc, int argc, char *argv[])
 {
 	if (argc < 4)
-		UT_FATAL("usage: test_remove <target> <pool set> "
-			"<force> <rm pool set>");
+		UT_FATAL(
+			"usage: test_remove <target> <pool set> <force> <rm pool set>");
 
 	const char *target = argv[0];
 	const char *pool_set = argv[1];
@@ -635,25 +776,12 @@ test_set_attr(const struct test_case *tc, int argc, char *argv[])
 	return 3;
 }
 
-/*
- * check_pool -- check if remote pool contains specified random sequence
- */
-static int
-check_pool(const struct test_case *tc, int argc, char *argv[])
+static void
+check_range_imp(char *pool_set, size_t offset, size_t size)
 {
-	if (argc < 3)
-		UT_FATAL("usage: fill_pool <pool set> <seed> <size>");
-
-	char *pool_set = argv[0];
-	srand(atoi(argv[1]));
-
+	struct pool_set *set;
 	int ret;
 
-	size_t size;
-	ret = util_parse_size(argv[2], &size);
-	size -= POOL_HDR_SIZE;
-
-	struct pool_set *set;
 	ret = util_poolset_create_set(&set, pool_set, 0, 0, 0);
 	UT_ASSERTeq(ret, 0);
 	ret = util_pool_open_nocheck(set, 0);
@@ -662,10 +790,58 @@ check_pool(const struct test_case *tc, int argc, char *argv[])
 	uint8_t *data = set->replica[0]->part[0].addr;
 	for (size_t i = 0; i < size; i++) {
 		uint8_t r = rand();
-		UT_ASSERTeq(data[POOL_HDR_SIZE + i], r);
+		UT_ASSERTeq(data[offset + i], r);
 	}
 
 	util_poolset_close(set, DO_NOT_DELETE_PARTS);
+}
+
+/*
+ * check_range -- check if remote pool range contains specified random sequence
+ */
+static int
+check_range(const struct test_case *tc, int argc, char *argv[])
+{
+	if (argc < 4)
+		UT_FATAL(
+			"usage: check_range <pool set> <seed> <offset> <size>");
+
+	char *pool_set = argv[0];
+	srand(ATOU(argv[1]));
+
+	size_t offset;
+	size_t size;
+	int ret;
+
+	ret = util_parse_size(argv[2], &offset);
+	UT_ASSERTeq(ret, 0);
+	ret = util_parse_size(argv[3], &size);
+	UT_ASSERTeq(ret, 0);
+
+	check_range_imp(pool_set, offset, size);
+
+	return 4;
+}
+
+/*
+ * check_pool -- check if remote pool contains specified random sequence
+ */
+static int
+check_pool(const struct test_case *tc, int argc, char *argv[])
+{
+	if (argc < 3)
+		UT_FATAL("usage: check_pool <pool set> <seed> <size>");
+
+	char *pool_set = argv[0];
+	srand(ATOU(argv[1]));
+
+	size_t size;
+	int ret;
+
+	ret = util_parse_size(argv[2], &size);
+	UT_ASSERTeq(ret, 0);
+
+	check_range_imp(pool_set, POOL_HDR_SIZE, size - POOL_HDR_SIZE);
 
 	return 3;
 }
@@ -680,7 +856,7 @@ fill_pool(const struct test_case *tc, int argc, char *argv[])
 		UT_FATAL("usage: fill_pool <pool set> <seed>");
 
 	char *pool_set = argv[0];
-	srand(atoi(argv[1]));
+	srand(ATOU(argv[1]));
 
 	int ret;
 
@@ -699,12 +875,60 @@ fill_pool(const struct test_case *tc, int argc, char *argv[])
 	return 2;
 }
 
+/*
+ * buff_limit -- limit working buffer
+ */
+static int
+buff_limit(const struct test_case *tc, int argc, char *argv[])
+{
+	if (argc < 3)
+		UT_FATAL("usage: buff_limit <id> <offset> <length>");
+
+	unsigned id = ATOU(argv[0]);
+	UT_ASSERT(id >= 0 && id < MAX_IDS);
+	size_t offset;
+	size_t size;
+	int ret;
+
+	ret = util_parse_size(argv[1], &offset);
+	UT_ASSERTeq(ret, 0);
+	ret = util_parse_size(argv[2], &size);
+	UT_ASSERTeq(ret, 0);
+
+	struct pool_entry *pool = &pools[id];
+	UT_ASSERT(offset < pool->size);
+	UT_ASSERT(offset + size <= pool->size);
+
+	pool->buff_offset = offset;
+	pool->buff_size = size;
+
+	return 3;
+}
+
+/*
+ * buff_reset -- reset working buffer to default (whole pool)
+ */
+static int
+buff_reset(const struct test_case *tc, int argc, char *argv[])
+{
+	if (argc < 1)
+		UT_FATAL("usage: buff_reset <id>");
+
+	unsigned id = ATOU(argv[0]);
+	UT_ASSERT(id >= 0 && id < MAX_IDS);
+
+	struct pool_entry *pool = &pools[id];
+	init_buff(pool);
+
+	return 1;
+}
+
 enum wait_type {
 	WAIT,
 	NOWAIT
 };
 
-const char *wait_type_str[2] = {
+static const char *wait_type_str[2] = {
 	"wait",
 	"nowait"
 };
@@ -800,8 +1024,8 @@ static int
 rpmemd_terminate(const struct test_case *tc, int argc, char *argv[])
 {
 	if (argc < 3) {
-		UT_FATAL("usage: rpmemd_terminate <id> <pid file> "
-				"<wait|nowait>");
+		UT_FATAL(
+			"usage: rpmemd_terminate <id> <pid file> <wait|nowait>");
 	}
 
 	const char *id_str = argv[0];
@@ -835,11 +1059,17 @@ rpmemd_terminate(const struct test_case *tc, int argc, char *argv[])
 static int
 test_persist_header(const struct test_case *tc, int argc, char *argv[])
 {
-	if (argc < 2)
-		UT_FATAL("usage: test_persist_header <id> <hdr|nohdr>");
+	if (argc < 3)
+		UT_FATAL(
+			"usage: test_persist_header <id> <hdr|nohdr> <relaxed>");
 
 	int id = atoi(argv[0]);
 	const char *hdr_str = argv[1];
+	int relaxed = atoi(argv[2]);
+	unsigned flags = 0;
+	if (relaxed)
+		flags |= RPMEM_PERSIST_RELAXED;
+
 	UT_ASSERT(id >= 0 && id < MAX_IDS);
 	struct pool_entry *pool = &pools[id];
 
@@ -852,11 +1082,11 @@ test_persist_header(const struct test_case *tc, int argc, char *argv[])
 		UT_ASSERT(0);
 
 	for (size_t off = 0; off < POOL_HDR_SIZE; off += 8) {
-		int ret = rpmem_persist(pool->rpp, off, 8, 0);
+		int ret = rpmem_persist(pool->rpp, off, 8, 0, flags);
 		UT_ASSERTeq(ret, with_hdr ? -1 : 0);
 	}
 
-	return 2;
+	return 3;
 }
 
 /*
@@ -869,10 +1099,15 @@ static struct test_case test_cases[] = {
 	TEST_CASE(test_close),
 	TEST_CASE(test_persist),
 	TEST_CASE(test_deep_persist),
+	TEST_CASE(test_flush),
+	TEST_CASE(test_drain),
 	TEST_CASE(test_read),
 	TEST_CASE(test_remove),
 	TEST_CASE(check_pool),
+	TEST_CASE(check_range),
 	TEST_CASE(fill_pool),
+	TEST_CASE(buff_limit),
+	TEST_CASE(buff_reset),
 	TEST_CASE(rpmemd_terminate),
 	TEST_CASE(test_persist_header),
 };
