@@ -1,5 +1,5 @@
 #
-# Copyright 2014-2019, Intel Corporation
+# Copyright 2014-2020, Intel Corporation
 # Copyright (c) 2016, Microsoft Corporation. All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -36,7 +36,50 @@ set -e
 export LC_ALL="C"
 #export LC_ALL="en_US.UTF-8"
 
+if ! [ -f ../envconfig.sh ]; then
+	echo >&2 "envconfig.sh is missing -- is the tree built?"
+	exit 1
+fi
+
 . ../testconfig.sh
+. ../envconfig.sh
+
+if [ -t 1 ]; then
+	IS_TERMINAL_STDOUT=YES
+fi
+if [ -t 2 ]; then
+	IS_TERMINAL_STDERR=YES
+fi
+
+function is_terminal() {
+	local fd
+	fd=$1
+	case $(eval "echo \${IS_TERMINAL_${fd}}") in
+	YES) : ;;
+	*) false ;;
+	esac
+}
+
+function interactive_color() {
+	local color fd
+	color=$1
+	fd=$2
+	shift 2
+
+	if is_terminal ${fd} && command -v tput >/dev/null; then
+		echo "$(tput setaf $color || :)$*$(tput sgr0 || :)"
+	else
+		echo "$*"
+	fi
+}
+
+function interactive_red() {
+	interactive_color 1 "$@"
+}
+
+function interactive_green() {
+	interactive_color 2 "$@"
+}
 
 function verbose_msg() {
 	if [ "$UNITTEST_LOG_LEVEL" -ge 2 ]; then
@@ -76,7 +119,6 @@ fi
 
 export UNITTEST_LOG_LEVEL GREP TEST FS BUILD CHECK_TYPE CHECK_POOL VERBOSE SUFFIX
 
-VMMALLOC=libvmmalloc.so.1
 TOOLS=../tools
 LIB_TOOLS="../../tools"
 # Paths to some useful tools
@@ -104,7 +146,7 @@ shopt -s failglob
 # number of remote nodes required in the current unit test
 NODES_MAX=-1
 
-# sizes of aligments
+# sizes of alignments
 SIZE_4KB=4096
 SIZE_2MB=2097152
 
@@ -178,8 +220,8 @@ nondebug|static-nondebug)
 	;;
 esac
 
-export LD_LIBRARY_PATH=$PMDK_LIB_PATH:$LIBNDCTL_LD_LIBRARY_PATHS:$LD_LIBRARY_PATH
-export REMOTE_LD_LIBRARY_PATH=$REMOTE_PMDK_LIB_PATH:$LIBNDCTL_LD_LIBRARY_PATHS:\$LD_LIBRARY_PATH
+export LD_LIBRARY_PATH=$PMDK_LIB_PATH:$GLOBAL_LIB_PATH:$LD_LIBRARY_PATH
+export REMOTE_LD_LIBRARY_PATH=$REMOTE_PMDK_LIB_PATH:$GLOBAL_LIB_PATH:\$LD_LIBRARY_PATH
 
 #
 # When running static binary tests, append the build type to the binary
@@ -268,8 +310,6 @@ fi
 # The default is to turn on library logging to level 3 and save it to local files.
 # Tests that don't want it on, should override these environment variables.
 #
-export VMEM_LOG_LEVEL=3
-export VMEM_LOG_FILE=vmem$UNITTEST_NUM.log
 export PMEM_LOG_LEVEL=3
 export PMEM_LOG_FILE=pmem$UNITTEST_NUM.log
 export PMEMBLK_LOG_LEVEL=3
@@ -282,10 +322,6 @@ export PMEMPOOL_LOG_LEVEL=3
 export PMEMPOOL_LOG_FILE=pmempool$UNITTEST_NUM.log
 export PMREORDER_LOG_FILE=pmreorder$UNITTEST_NUM.log
 
-export VMMALLOC_POOL_SIZE=$((16 * 1024 * 1024))
-export VMMALLOC_LOG_LEVEL=3
-export VMMALLOC_LOG_FILE=vmmalloc$UNITTEST_NUM.log
-
 export OUT_LOG_FILE=out$UNITTEST_NUM.log
 export ERR_LOG_FILE=err$UNITTEST_NUM.log
 export TRACE_LOG_FILE=trace$UNITTEST_NUM.log
@@ -444,7 +480,7 @@ function create_file() {
 	shift
 	for file in $*
 	do
-		$DD if=/dev/zero of=$file bs=1M count=$size iflag=count_bytes >> $PREP_LOG_FILE
+		$DD if=/dev/zero of=$file bs=1M count=$size iflag=count_bytes status=none >> $PREP_LOG_FILE
 	done
 }
 
@@ -662,6 +698,21 @@ function valgrind_ignore_warnings() {
 	mv $1.tmp $1
 }
 
+#
+# valgrind_ignore_messages -- cuts off Valgrind messages that are irrelevant
+#	to the correctness of the test, but changes during Valgrind rebase
+#	usage: valgrind_ignore_messages <log-file>
+#
+function valgrind_ignore_messages() {
+	if [ -e "$1.match" ]; then
+		cat $1 | grep -v \
+			-e "For lists of detected and suppressed errors, rerun with: -s" \
+			-e "For counts of detected and suppressed errors, rerun with: -v" \
+			>  $1.tmp
+		mv $1.tmp $1
+	fi
+}
+
 #
 # get_trace -- return tracing tool command line if applicable
 #	usage: get_trace <check type> <log file> [<node>]
@@ -729,8 +780,7 @@ function validate_valgrind_log() {
 		-e "Bad mempool" \
 		$1 >/dev/null ;
 	then
-		msg="failed"
-		[ -t 2 ] && command -v tput >/dev/null && msg="$(tput setaf 1)$msg$(tput sgr0)"
+		msg=$(interactive_red STDERR "failed")
 		echo -e "$UNITTEST_NAME $msg with Valgrind. See $1. Last 20 lines below." >&2
 		paste -d " " <(yes $UNITTEST_NAME $1 | head -n 20) <(tail -n 20 $1) >&2
 		false
@@ -784,16 +834,6 @@ function expect_normal_exit() {
 		export VALGRIND_OPTS="$VALGRIND_OPTS --suppressions=../memcheck-dlopen.supp"
 	fi
 
-	# in case of preloading libvmmalloc.so.1 force valgrind to not override malloc
-	if [ -n "$VALGRINDEXE" -a -n "$TEST_LD_PRELOAD" ]; then
-		if [ $(valgrind_version) -ge 312 ]; then
-			preload=`basename $TEST_LD_PRELOAD`
-		fi
-		if [ "$preload" == "$VMMALLOC" ]; then
-			export VALGRIND_OPTS="$VALGRIND_OPTS --soname-synonyms=somalloc=nouserintercepts"
-		fi
-	fi
-
 	local REMOTE_VALGRIND_LOG=0
 	if [ "$CHECK_TYPE" != "none" ]; then
 	        case "$1"
@@ -821,7 +861,7 @@ function expect_normal_exit() {
 
 	disable_exit_on_error
 
-	eval $ECHO LD_PRELOAD=$TEST_LD_PRELOAD $trace "$*"
+	eval $ECHO $trace "$*"
 	ret=$?
 
 	if [ $REMOTE_VALGRIND_LOG -eq 1 ]; then
@@ -840,7 +880,7 @@ function expect_normal_exit() {
 		else
 			msg="failed with exit code $ret"
 		fi
-		[ -t 2 ] && command -v tput >/dev/null && msg="$(tput setaf 1)$msg$(tput sgr0)"
+		msg=$(interactive_red STDERR $msg)
 
 		if [ -f $ERR_LOG_FILE ]; then
 			if [ "$UNITTEST_LOG_LEVEL" -ge "1" ]; then
@@ -870,11 +910,13 @@ function expect_normal_exit() {
 			do
 				local log_file=node\_$node\_$VALGRIND_LOG_FILE
 				valgrind_ignore_warnings $new_log_file
+				valgrind_ignore_messages $new_log_file
 				validate_valgrind_log $new_log_file
 			done
 		else
 			if [ -f $VALGRIND_LOG_FILE ]; then
 				valgrind_ignore_warnings $VALGRIND_LOG_FILE
+				valgrind_ignore_messages $VALGRIND_LOG_FILE
 				validate_valgrind_log $VALGRIND_LOG_FILE
 			fi
 		fi
@@ -898,29 +940,17 @@ function expect_abnormal_exit() {
 		esac
 	fi
 
-	# in case of preloading libvmmalloc.so.1 force valgrind to not override malloc
-	if [ -n "$VALGRINDEXE" -a -n "$TEST_LD_PRELOAD" ]; then
-		if [ $(valgrind_version) -ge 312 ]; then
-			preload=`basename $TEST_LD_PRELOAD`
-		fi
-		if [ "$preload" == "$VMMALLOC" ]; then
-			export VALGRIND_OPTS="$VALGRIND_OPTS --soname-synonyms=somalloc=nouserintercepts"
-		fi
-	fi
-
 	if [ "$CHECK_TYPE" = "drd" ]; then
 		export VALGRIND_OPTS="$VALGRIND_OPTS --suppressions=../drd-log.supp"
 	fi
 
 	disable_exit_on_error
-	eval $ECHO ASAN_OPTIONS="detect_leaks=0 ${ASAN_OPTIONS}" \
-		LD_PRELOAD=$TEST_LD_PRELOAD $TRACE "$*"
+	eval $ECHO ASAN_OPTIONS="detect_leaks=0 ${ASAN_OPTIONS}" $TRACE "$*"
 	ret=$?
 	restore_exit_on_error
 
 	if [ "$ret" -eq "0" ]; then
-		msg="succeeded"
-		[ -t 2 ] && command -v tput >/dev/null && msg="$(tput setaf 1)$msg$(tput sgr0)"
+		msg=$(interactive_red STDERR "succeeded")
 
 		echo -e "$UNITTEST_NAME command $msg unexpectedly." >&2
 
@@ -993,7 +1023,7 @@ function require_sudo_allowed() {
 		exit 0
 	fi
 
-	if ! timeout --signal=SIGKILL --kill-after=3s 3s sudo date >/dev/null 2>&1
+	if ! sh -c "timeout --signal=SIGKILL --kill-after=3s 3s sudo date" >/dev/null 2>&1
 	then
 		msg "$UNITTEST_NAME: SKIP required: sudo allowed"
 		exit 0
@@ -1046,14 +1076,50 @@ function require_procfs() {
 	exit 0
 }
 
-function get_arch() {
-	gcc -dumpmachine | awk -F'[/-]' '{print $1}'
+#
+# require_arch -- Skip tests if the running platform not matches
+# any of the input list.
+#
+function require_arch() {
+	for i in "$@"; do
+		[[ "$(arch)" == "$i" ]] && return
+	done
+	msg "$UNITTEST_NAME: SKIP: Only supported on $1"
+	exit 0
 }
 
+#
+# exclude_arch -- Skip tests if the running platform matches
+# any of the input list.
+#
+function exclude_arch() {
+	for i in "$@"; do
+		if [[ "$(arch)" == "$i" ]]; then
+			msg "$UNITTEST_NAME: SKIP: Not supported on $1"
+			exit 0
+		fi
+	done
+}
+
+#
+# require_x86_64 -- Skip tests if the running platform is not x86_64
+#
 function require_x86_64() {
-	[ $(get_arch) = "x86_64" ] && return
-	msg "$UNITTEST_NAME: SKIP: Not supported on arch != x86_64"
-	exit 0
+	require_arch x86_64
+}
+
+#
+# require_ppc64 -- Skip tests if the running platform is not ppc64 or ppc64le
+#
+function require_ppc64() {
+	require_arch "ppc64" "ppc64le" "ppc64el"
+}
+
+#
+# exclude_ppc64 -- Skip tests if the running platform is ppc64 or ppc64le
+#
+function exclude_ppc64() {
+	exclude_arch "ppc64" "ppc64le" "ppc64el"
 }
 
 #
@@ -1304,18 +1370,6 @@ function get_node_devdax_size() {
 	echo $out
 }
 
-#
-# dax_get_alignment -- get the alignment of a device dax
-#
-function dax_get_alignment() {
-	major_hex=$(stat -c "%t" $1)
-	minor_hex=$(stat -c "%T" $1)
-	major_dec=$((16#$major_hex))
-	minor_dec=$((16#$minor_hex))
-	cat /sys/dev/char/$major_dec:$minor_dec/device/align
-}
-
-
 #
 # require_dax_device_node_alignments -- only allow script to continue if
 #    the internal Device DAX alignments on a remote nodes are as specified.
@@ -1400,6 +1454,18 @@ require_dax_device_alignments() {
 	require_node_dax_device_alignments -1 $*
 }
 
+#
+# disable_eatmydata -- ensure invalid msyncs fail
+#
+# Distros (and people) like to use eatmydata to kill fsync-likes during builds
+# and testing. This is nice for speed, but we actually rely on msync failing
+# in some tests.
+#
+disable_eatmydata() {
+	export LD_PRELOAD="${LD_PRELOAD/#libeatmydata.so/}"
+	export LD_PRELOAD="${LD_PRELOAD/ libeatmydata.so/}"
+	export LD_PRELOAD="${LD_PRELOAD/:libeatmydata.so/}"
+}
 
 #
 # require_fs_type -- only allow script to continue for a certain fs type
@@ -1417,7 +1483,6 @@ function require_fs_type() {
 	exit 0
 }
 
-
 #
 # require_native_fallocate -- verify if filesystem supports fallocate
 #
@@ -1909,29 +1974,6 @@ function require_binary() {
 	return
 }
 
-#
-# require_preload - continue script execution only if supplied
-#	executable does not generate SIGABRT
-#
-#	Used to check that LD_PRELOAD of, e.g., libvmmalloc is possible
-#
-#	usage: require_preload <errorstr> <executable> [<exec_args>]
-#
-function require_preload() {
-	msg=$1
-	shift
-	trap SIGABRT
-	disable_exit_on_error
-	ret=$(LD_PRELOAD=$TEST_LD_PRELOAD $* 2>&1 /dev/null)
-	ret=$?
-	restore_exit_on_error
-	if [ $ret == 134 ]; then
-		msg "$UNITTEST_NAME: SKIP: $msg not supported"
-		rm -f $1.core
-		exit 0
-	fi
-}
-
 #
 # require_sds -- continue script execution only if binary is compiled with
 #	shutdown state support
@@ -1977,32 +2019,32 @@ function require_no_sds() {
 }
 
 #
-# is_ndctl_ge_63 -- check if binary is compiled with libndctl 63+
+# is_ndctl_enabled -- check if binary is compiled with libndctl
 #
-#	usage: is_ndctl_ge_63 <binary>
+#	usage: is_ndctl_enabled <binary>
 #
-function is_ndctl_ge_63() {
+function is_ndctl_enabled() {
 	local binary=$1
 	local dir=.
 	if [ -z "$binary" ]; then
-		fatal "is_ndctl_ge_63: error: no binary found"
+		fatal "is_ndctl_enabled: error: no binary found"
 	fi
 
 	strings ${binary} 2>&1 | \
-		grep -q "compiled with libndctl 63+" && true
+		grep -q "compiled with libndctl" && true
 
 	return $?
 }
 
 #
-# require_user_bb -- checks if the binary has support for unprivileged
-#	bad block iteration
+# require_bb_enabled_by_default -- check if the binary has bad block
+#                                     checking feature enabled by default
 #
-#	usage: require_user_bb <binary>
+#	usage: require_bb_enabled_by_default <binary>
 #
-function require_user_bb() {
-	if ! is_ndctl_ge_63 $1 &> /dev/null ; then
-		msg "$UNITTEST_NAME: SKIP unprivileged bad block iteration not supported"
+function require_bb_enabled_by_default() {
+	if ! is_ndctl_enabled $1 &> /dev/null ; then
+		msg "$UNITTEST_NAME: SKIP bad block checking feature disabled by default"
 		exit 0
 	fi
 
@@ -2010,14 +2052,14 @@ function require_user_bb() {
 }
 
 #
-# require_su_bb -- checks if the binary does not have support for
-#	unprivileged bad block iteration
+# require_bb_disabled_by_default -- check if the binary does not have bad
+#                                      block checking feature enabled by default
 #
-#	usage: require_su_bb <binary>
+#	usage: require_bb_disabled_by_default <binary>
 #
-function require_su_bb() {
-	if is_ndctl_ge_63 $1 &> /dev/null ; then
-		msg "$UNITTEST_NAME: SKIP unprivileged bad block iteration supported"
+function require_bb_disabled_by_default() {
+	if is_ndctl_enabled $1 &> /dev/null ; then
+		msg "$UNITTEST_NAME: SKIP bad block checking feature enabled by default"
 		exit 0
 	fi
 	return 0
@@ -2047,12 +2089,10 @@ function run_command()
 	fi
 }
 
-
 #
 # validate_node_number -- validate a node number
 #
 function validate_node_number() {
-
 	[ $1 -gt $NODES_MAX ] \
 		&& fatal "error: node number ($1) greater than maximum allowed node number ($NODES_MAX)"
 	return 0
@@ -2139,7 +2179,10 @@ function require_node_libfabric() {
 	local version=${3:-1.4.2}
 
 	require_pkg libfabric "$version"
+	# fi_info can be found in libfabric-bin
+	require_command fi_info
 	require_node_pkg $N libfabric "$version"
+	require_command_node $N fi_info
 	if [ "$RPMEM_PROVIDER" == "verbs" ]; then
 		if ! fi_info --list | grep -q verbs; then
 			msg "$UNITTEST_NAME: SKIP libfabric not compiled with verbs provider"
@@ -2535,6 +2578,39 @@ function kill_on_node() {
 	return $ret
 }
 
+#
+# obj_pool_desc_size -- returns the obj_pool_desc_size macro value
+# in bytes which is two times the actual pagesize.
+#
+# This should be use to calculate the minimum zero size for pool
+# creation on some tests.
+#
+function obj_pool_desc_size() {
+	echo "$(expr $(getconf PAGESIZE) \* 2)"
+}
+
+#
+# log_pool_desc_size -- returns the minimum size of pool header
+# in bytes which is two times the actual pagesize.
+#
+# This should be use to calculate the minimum zero size for pool
+# creation on some tests.
+#
+function log_pool_desc_size() {
+	echo "$(expr $(getconf PAGESIZE) \* 2)"
+}
+
+#
+# blk_pool_desc_size -- returns the minimum size of pool header
+# in bytes which is two times the actual pagesize.
+#
+# This should be use to calculate the minimum zero size for pool
+# creation on some tests.
+#
+function blk_pool_desc_size() {
+	echo "$(expr $(getconf PAGESIZE) \* 2)"
+}
+
 #
 # create_holey_file_on_node -- create holey files of a given length
 #   usage: create_holey_file_on_node <node> <size>
@@ -2588,7 +2664,6 @@ function require_mmap_under_valgrind() {
 function setup() {
 
 	DIR=$DIR$SUFFIX
-	export VMMALLOC_POOL_DIR="$DIR"
 
 	# writes test working directory to temporary file
 	# that allows read location of data after test failure
@@ -2753,8 +2828,7 @@ function pass() {
 	else
 		tm=""
 	fi
-	msg="PASS"
-	[ -t 1 ] && command -v tput >/dev/null && msg="$(tput setaf 2)$msg$(tput sgr0)"
+	msg=$(interactive_green STDOUT "PASS")
 	if [ "$UNITTEST_LOG_LEVEL" -ge 1 ]; then
 		echo -e "$UNITTEST_NAME: $msg$tm"
 	fi
@@ -2767,7 +2841,7 @@ function pass() {
 SIG_LEN=8
 
 # Offset and length of pmemobj layout
-LAYOUT_OFFSET=4096
+LAYOUT_OFFSET=$(getconf PAGE_SIZE)
 LAYOUT_LEN=1024
 
 # Length of arena's signature
@@ -2777,7 +2851,7 @@ ARENA_SIG_LEN=16
 ARENA_SIG="BTT_ARENA_INFO"
 
 # Offset to first arena
-ARENA_OFF=8192
+ARENA_OFF=$(($(getconf PAGE_SIZE) * 2))
 
 #
 # check_file -- check if file exists and print error message if not
@@ -3439,8 +3513,7 @@ function pmreorder_expect_success()
 	ret=$(pmreorder_run_tool "$@")
 
 	if [ "$ret" -ne "0" ]; then
-		msg="failed with exit code $ret"
-		[ -t 2 ] && command -v tput >/dev/null && msg="$(tput setaf 1)$msg$(tput sgr0)"
+		msg=$(interactive_red STDERR "failed with exit code $ret")
 
 		# exit code 130 - script terminated by user (Control-C)
 		if [ "$ret" -ne "130" ]; then
@@ -3462,8 +3535,7 @@ function pmreorder_expect_failure()
 	ret=$(pmreorder_run_tool "$@")
 
 	if [ "$ret" -eq "0" ]; then
-		msg="succeeded"
-		[ -t 2 ] && command -v tput >/dev/null && msg="$(tput setaf 1)$msg$(tput sgr0)"
+		msg=$(interactive_red STDERR "succeeded")
 
 		echo -e "$UNITTEST_NAME command $msg unexpectedly." >&2
 
@@ -3556,15 +3628,85 @@ function require_max_devdax_size() {
 }
 
 #
-# require_nfit_tests_enabled - check if tests using the nfit_test kernel module are not enabled
+# require_max_block_size -- checks that block size is smaller or equal than requested
 #
-function require_nfit_tests_enabled() {
-	if [ "$ENABLE_NFIT_TESTS" != "y" ]; then
-		msg "$UNITTEST_NAME: SKIP: tests using the nfit_test kernel module are not enabled in testconfig.sh (ENABLE_NFIT_TESTS)"
+# usage: require_max_block_size <file> <max-block-size>
+#
+function require_max_block_size() {
+	cur_sz=$(stat --file-system --format=%S $1)
+	max_size=$2
+	if [ $cur_sz -gt $max_size ]; then
+		msg "$UNITTEST_NAME: SKIP: block size of $1 is too big for this test (max $2 required)"
 		exit 0
 	fi
 }
 
+#
+# require_badblock_tests_enabled - check if tests for bad block support are not enabled
+# Input arguments:
+# 1) test device type
+#
+function require_badblock_tests_enabled() {
+	require_sudo_allowed
+	require_command ndctl
+	require_bb_enabled_by_default $PMEMPOOL$EXESUFFIX
+
+	if [ "$BADBLOCK_TEST_TYPE" == "nfit_test" ]; then
+
+		require_kernel_module nfit_test
+
+		# nfit_test dax device is created by the test and is
+		# used directly - no device dax path is needed to be provided by the
+		# user. Some tests though may use an additional filesystem for the
+		# pool replica - hence 'any' filesystem is required.
+		if [ $1 == "dax_device" ]; then
+			require_fs_type any
+
+		# nfit_test block device is created by the test and mounted on
+		# a filesystem of any type provided by the user
+		elif [ $1 == "block_device" ]; then
+			require_fs_type any
+		fi
+
+	elif [ "$BADBLOCK_TEST_TYPE" == "real_pmem" ]; then
+
+		if [ $1 == "dax_device" ]; then
+			require_fs_type any
+			require_dax_devices 1
+			require_binary $DAXIO$EXESUFFIX
+
+		elif [ $1 == "block_device" ]; then
+			require_fs_type pmem
+		fi
+
+	else
+		msg "$UNITTEST_NAME: SKIP: bad block tests are not enabled in testconfig.sh"
+		exit 0
+	fi
+}
+
+#
+# require_badblock_tests_enabled_node - check if tests for bad block support are not enabled
+# on given remote node
+#
+function require_badblock_tests_enabled_node() {
+	require_sudo_allowed_node $1
+	require_command_node $1 ndctl
+	require_bb_enabled_by_default $PMEMPOOL$EXESUFFIX
+
+	if [ "$BADBLOCK_TEST_TYPE" == "nfit_test" ]; then
+		require_kernel_module_node $1 nfit_test
+	elif [ "$BADBLOCK_TEST_TYPE" == "real_pmem" ]; then
+		:
+	else
+		msg "$UNITTEST_NAME: SKIP: bad block tests are not enabled in testconfig.sh"
+		exit 0
+	fi
+	require_sudo_allowed
+	require_kernel_module nfit_test
+	require_command ndctl
+}
+
 #
 # create_recovery_file - create bad block recovery file
 #
