@@ -1,5 +1,5 @@
 /*
- * Copyright 2014-2018, Intel Corporation
+ * Copyright 2014-2019, Intel Corporation
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -56,7 +56,6 @@
 #include "libpmemlog.h"
 #include "libpmempool.h"
 
-
 #define DEFAULT_MODE	0664
 /*
  * pmempool_create -- context and args for create command
@@ -115,7 +114,7 @@ static const char * const help_str =
 "  -M, --max-size       use maximum available space on file system\n"
 "  -m, --mode <octal>   set permissions to <octal> (the default is 0664)\n"
 "  -i, --inherit <file> take required parameters from specified pool file\n"
-"  -b, --clearbadblocks clear bad blocks in existing files\n"
+"  -b, --clear-bad-blocks clear bad blocks in existing files\n"
 "  -f, --force          remove the pool first\n"
 "  -v, --verbose        increase verbosity level\n"
 "  -h, --help           display this help and exit\n"
@@ -142,7 +141,7 @@ static const struct option long_options[] = {
 	{"write-layout", no_argument,		NULL,	'w' | OPT_BLK},
 	{"layout",	required_argument,	NULL,	'l' | OPT_OBJ},
 	{"force",	no_argument,		NULL,	'f' | OPT_ALL},
-	{"clearbadblocks", no_argument,		NULL,	'b' | OPT_ALL},
+	{"clear-bad-blocks", no_argument,		NULL,	'b' | OPT_ALL},
 	{NULL,		0,			NULL,	 0 },
 };
 
@@ -200,13 +199,9 @@ pmempool_create_obj(struct pmempool_create *pcp)
 static int
 pmempool_create_blk(struct pmempool_create *pcp)
 {
-	int ret = 0;
+	ASSERTne(pcp->params.blk.bsize, 0);
 
-	if (pcp->params.blk.bsize == 0) {
-		outv(1, "No block size option passed"
-				" - picking minimum block size.\n");
-		pcp->params.blk.bsize = PMEMBLK_MIN_BLK;
-	}
+	int ret = 0;
 
 	PMEMblkpool *pbp = pmemblk_create(pcp->fname, pcp->params.blk.bsize,
 			pcp->params.size, pcp->params.mode);
@@ -436,6 +431,42 @@ pmempool_create_parse_args(struct pmempool_create *pcp, const char *appname,
 	return 0;
 }
 
+static int
+allocate_max_size_available_file(const char *name_of_file, mode_t mode,
+		os_off_t max_size)
+{
+	int fd = os_open(name_of_file, O_CREAT | O_EXCL | O_RDWR, mode);
+	if (fd == -1) {
+		outv_err("!open '%s' failed", name_of_file);
+		return -1;
+	}
+
+	os_off_t offset = 0;
+	os_off_t length = max_size - (max_size % (os_off_t)Pagesize);
+	int ret;
+	do {
+		ret = os_posix_fallocate(fd, offset, length);
+		if (ret == 0)
+			offset += length;
+		else if (ret != ENOSPC) {
+			os_close(fd);
+			if (os_unlink(name_of_file) == -1)
+				outv_err("!unlink '%s' failed", name_of_file);
+			errno = ret;
+			outv_err("!space allocation for '%s' failed",
+					name_of_file);
+			return -1;
+		}
+
+		length /= 2;
+		length -= (length % (os_off_t)Pagesize);
+	} while (length > (os_off_t)Pagesize);
+
+	os_close(fd);
+
+	return 0;
+}
+
 /*
  * pmempool_create_func -- main function for create command
  */
@@ -499,6 +530,21 @@ pmempool_create_func(const char *appname, int argc, char *argv[])
 				return -1;
 			}
 		}
+
+		if (PMEM_POOL_TYPE_OBJ == pc.params.type && pc.layout != NULL) {
+			size_t max_layout = PMEMOBJ_MAX_LAYOUT;
+
+			if (strlen(pc.layout) >= max_layout) {
+				outv_err(
+						"Layout name is too long, maximum number of characters (including the terminating null byte) is %zu\n",
+						max_layout);
+				return -1;
+			}
+
+			size_t len = sizeof(pc.params.obj.layout);
+			strncpy(pc.params.obj.layout, pc.layout, len);
+			pc.params.obj.layout[len - 1] = '\0';
+		}
 	} else if (pc.inherit_fname) {
 		pc.params.type = pc.inherit_params.type;
 	} else {
@@ -537,15 +583,6 @@ pmempool_create_func(const char *appname, int argc, char *argv[])
 		return -1;
 	}
 
-	size_t max_layout = PMEMOBJ_MAX_LAYOUT;
-
-	if (pc.layout && strlen(pc.layout) >= max_layout) {
-		outv_err("Layout name is to long, maximum number of characters"
-			" (including the terminating null byte) is %zu\n",
-			max_layout);
-		return -1;
-	}
-
 	if (pc.inherit_fname)  {
 		if (!pc.str_size && !pc.max_size)
 			pc.params.size = pc.inherit_params.size;
@@ -595,6 +632,15 @@ pmempool_create_func(const char *appname, int argc, char *argv[])
 			}
 			outv(1, "Available space is %s\n",
 				out_get_size_str(pc.params.size, 2));
+			if (allocate_max_size_available_file(pc.fname,
+					pc.params.mode,
+					(os_off_t)pc.params.size))
+				return -1;
+			/*
+			 * We are going to create pool based
+			 * on file size instead of the pc.params.size.
+			 */
+			pc.params.size = 0;
 		} else {
 			if (!pc.fexists) {
 				outv(1, "No size option passed "
