@@ -1,6 +1,6 @@
 #!/usr/bin/env bash
 #
-# Copyright 2014-2019, Intel Corporation
+# Copyright 2014-2020, Intel Corporation
 #
 # Redistribution and use in source and binary forms, with or without
 # modification, are permitted provided that the following conditions
@@ -49,16 +49,18 @@ usage()
 Usage: $0 [ -h ] -t version-tag -s source-dir -w working-dir -o output-dir
 	[ -e build-experimental ] [ -c run-check ]
 	[ -n with-ndctl ] [ -f testconfig-file ]
+	[ -p build-libpmem2 ]
 
 -h			print this help message
 -t version-tag		source version tag
 -s source-dir		source directory
 -w working-dir		working directory
--o output-dir		outut directory
+-o output-dir		output directory
 -e build-experimental	build experimental packages
 -c run-check		run package check
 -n with-ndctl		build with libndctl
 -f testconfig-file	custom testconfig.sh
+-p build-libpmem2	build libpmem2 packages
 EOF
 	exit 1
 }
@@ -66,7 +68,7 @@ EOF
 #
 # command-line argument processing...
 #
-args=`getopt he:c:r:n:t:d:s:w:o:f: $*`
+args=`getopt he:c:r:n:t:d:s:w:o:f:p: $*`
 [ $? != 0 ] && usage
 set -- $args
 for arg
@@ -110,6 +112,10 @@ do
 		OUT_DIR="$2"
 		shift 2
 		;;
+	-p)
+		PMEM2_INSTALL="$2"
+		shift 2
+		;;
 	--)
 		shift
 		break
@@ -117,7 +123,6 @@ do
 	esac
 done
 
-
 # check for mandatory arguments
 if [ -z "$PACKAGE_VERSION_TAG" -o -z "$SOURCE" -o -z "$WORKING_DIR" -o -z "$OUT_DIR" ]
 then
@@ -125,7 +130,6 @@ then
 	usage
 fi
 
-
 PREFIX=usr
 LIB_DIR=$PREFIX/lib/$(dpkg-architecture -qDEB_HOST_MULTIARCH)
 INC_DIR=$PREFIX/include
@@ -192,7 +196,7 @@ cat << EOF > debian/librpmem-dev.lintian-overrides
 $ITP_BUG_EXCUSE
 new-package-should-close-itp-bug
 # The following warnings are triggered by a bug in debhelper:
-# http://bugs.debian.org/204975
+# https://bugs.debian.org/204975
 postinst-has-useless-call-to-ldconfig
 postrm-has-useless-call-to-ldconfig
 # We do not want to compile with -O2 for debug version
@@ -219,7 +223,7 @@ cat << EOF >> $CONTROL_FILE
 
 Package: librpmem
 Architecture: any
-Depends: libfabric (>= $LIBFABRIC_MIN_VERSION), \${shlibs:Depends}, \${misc:Depends}
+Depends: \${shlibs:Depends}, \${misc:Depends}
 Description: Persistent Memory remote access support library
  librpmem provides low-level support for remote access to persistent memory
  (pmem) utilizing RDMA-capable RNICs. The library can be used to replicate
@@ -248,12 +252,72 @@ Package: rpmemd
 Section: misc
 Architecture: any
 Priority: optional
-Depends: libfabric (>= $LIBFABRIC_MIN_VERSION), \${shlibs:Depends}, \${misc:Depends}
+Depends: \${shlibs:Depends}, \${misc:Depends}
 Description: rpmem daemon
  Daemon for Remote Persistent Memory support
 EOF
 }
 
+function libpmem2_install_triggers_overrides() {
+cat << EOF > debian/libpmem2.install
+$LIB_DIR/libpmem2.so.*
+EOF
+
+cat << EOF > debian/libpmem2.lintian-overrides
+$ITP_BUG_EXCUSE
+new-package-should-close-itp-bug
+libpmem2: package-name-doesnt-match-sonames
+EOF
+
+cat << EOF > debian/libpmem2-dev.install
+$LIB_DIR/pmdk_debug/libpmem2.a $LIB_DIR/pmdk_dbg/
+$LIB_DIR/pmdk_debug/libpmem2.so $LIB_DIR/pmdk_dbg/
+$LIB_DIR/pmdk_debug/libpmem2.so.* $LIB_DIR/pmdk_dbg/
+$LIB_DIR/libpmem2.so
+$LIB_DIR/pkgconfig/libpmem2.pc
+$INC_DIR/libpmem2.h
+$MAN7_DIR/libpmem2.7
+$MAN3_DIR/pmem2_*.3
+EOF
+
+cat << EOF > debian/libpmem2-dev.triggers
+interest man-db
+EOF
+
+cat << EOF > debian/libpmem2-dev.lintian-overrides
+$ITP_BUG_EXCUSE
+new-package-should-close-itp-bug
+# The following warnings are triggered by a bug in debhelper:
+# https://bugs.debian.org/204975
+postinst-has-useless-call-to-ldconfig
+postrm-has-useless-call-to-ldconfig
+# We do not want to compile with -O2 for debug version
+hardening-no-fortify-functions $LIB_DIR/pmdk_dbg/*
+EOF
+}
+
+function append_libpmem2_control() {
+cat << EOF >> $CONTROL_FILE
+
+Package: libpmem2
+Architecture: any
+Depends: \${shlibs:Depends}, \${misc:Depends}
+Description: Persistent Memory low level support library
+ libpmem2 provides low level persistent memory support. In particular, support
+ for the persistent memory instructions for flushing changes to pmem is
+ provided. (EXPERIMENTAL)
+
+Package: libpmem2-dev
+Section: libdevel
+Architecture: any
+Depends: libpmem2 (=\${binary:Version}), \${shlibs:Depends}, \${misc:Depends}
+Description: Development files for libpmem2
+ libpmem2 provides low level persistent memory support. In particular, support
+ for the persistent memory instructions for flushing changes to pmem is
+ provided. (EXPERIMENTAL)
+EOF
+}
+
 function daxio_install_triggers_overrides() {
 cat << EOF > debian/daxio.install
 usr/bin/daxio
@@ -288,7 +352,6 @@ Description: daxio utility
 EOF
 }
 
-
 if [ "${BUILD_PACKAGE_CHECK}" == "y" ]
 then
 CHECK_CMD="
@@ -339,6 +402,7 @@ tar zcf $PACKAGE_TARBALL_ORIG $PACKAGE_SOURCE
 
 cd $PACKAGE_SOURCE
 
+rm -rf debian
 mkdir debian
 
 # Generate compat file
@@ -354,7 +418,7 @@ Section: libs
 Priority: optional
 Standards-version: 4.1.4
 Build-Depends: debhelper (>= 9)
-Homepage: http://pmem.io/pmdk/
+Homepage: https://pmem.io/pmdk/
 
 Package: libpmem
 Architecture: any
@@ -445,56 +509,11 @@ Description: Development files for libpmempool
  This package contains libraries and header files used for linking programs
  against libpmempool.
 
-Package: libvmem
-Architecture: any
-Depends: \${shlibs:Depends}, \${misc:Depends}
-Description: Persistent Memory volatile memory support library
- The libvmem library turns a pool of persistent memory into a volatile memory
- pool, similar to the system heap but kept separate and with its own
- malloc-style API.
- .
- libvmem supports the traditional malloc/free interfaces on a memory mapped
- file. This allows the use of persistent memory as volatile memory, for cases
- where the pool of persistent memory is useful to an application, but when the
- application doesn’t need it to be persistent.
-
-Package: libvmem-dev
-Section: libdevel
-Architecture: any
-Depends: libvmem (=\${binary:Version}), \${shlibs:Depends}, \${misc:Depends}
-Description: Development files for libvmem
- The libvmem library turns a pool of persistent memory into a volatile memory
- pool, similar to the system heap but kept separate and with its own
- malloc-style API.
- .
- This package contains libraries and header files used for linking programs
- against libvmem.
-
-Package: libvmmalloc
-Architecture: any
-Depends: \${shlibs:Depends}, \${misc:Depends}
-Description: Persistent Memory dynamic allocation support library
- The libvmmalloc library transparently converts all the dynamic memory
- allocations into persistent memory allocations. This allows the use of
- persistent memory as volatile memory without modifying the target
- application.
-
-Package: libvmmalloc-dev
-Section: libdevel
-Architecture: any
-Depends: libvmmalloc (=\${binary:Version}), \${shlibs:Depends}, \${misc:Depends}
-Description: Development files for libvmmalloc
- The libvmmalloc library transparently converts all the dynamic memory
- allocations into persistent memory allocations.
- .
- This package contains libraries and header files used for linking programs
- against libvmalloc.
-
 Package: $PACKAGE_NAME-dbg
 Section: debug
 Priority: optional
 Architecture: any
-Depends: libvmem (=\${binary:Version}), libvmmalloc (=\${binary:Version}), libpmem (=\${binary:Version}), libpmemblk (=\${binary:Version}), libpmemlog (=\${binary:Version}), libpmemobj (=\${binary:Version}), libpmempool (=\${binary:Version}), \${misc:Depends}
+Depends: libpmem (=\${binary:Version}), libpmemblk (=\${binary:Version}), libpmemlog (=\${binary:Version}), libpmemobj (=\${binary:Version}), libpmempool (=\${binary:Version}), \${misc:Depends}
 Description: Debug symbols for PMDK libraries
  Debug symbols for all PMDK libraries.
 
@@ -540,10 +559,10 @@ override_dh_strip:
 	dh_strip --dbg-package=$PACKAGE_NAME-dbg
 
 override_dh_auto_build:
-	dh_auto_build -- EXPERIMENTAL=${EXPERIMENTAL} prefix=/$PREFIX libdir=/$LIB_DIR includedir=/$INC_DIR docdir=/$DOC_DIR man1dir=/$MAN1_DIR man3dir=/$MAN3_DIR man5dir=/$MAN5_DIR man7dir=/$MAN7_DIR sysconfdir=/etc bashcompdir=/usr/share/bash-completion/completions NORPATH=1 ${pass_ndctl_enable} SRCVERSION=$SRCVERSION
+	dh_auto_build -- EXPERIMENTAL=${EXPERIMENTAL} prefix=/$PREFIX libdir=/$LIB_DIR includedir=/$INC_DIR docdir=/$DOC_DIR man1dir=/$MAN1_DIR man3dir=/$MAN3_DIR man5dir=/$MAN5_DIR man7dir=/$MAN7_DIR sysconfdir=/etc bashcompdir=/usr/share/bash-completion/completions NORPATH=1 ${pass_ndctl_enable} SRCVERSION=$SRCVERSION PMEM2_INSTALL=${PMEM2_INSTALL}
 
 override_dh_auto_install:
-	dh_auto_install -- EXPERIMENTAL=${EXPERIMENTAL} prefix=/$PREFIX libdir=/$LIB_DIR includedir=/$INC_DIR docdir=/$DOC_DIR man1dir=/$MAN1_DIR man3dir=/$MAN3_DIR man5dir=/$MAN5_DIR man7dir=/$MAN7_DIR sysconfdir=/etc bashcompdir=/usr/share/bash-completion/completions NORPATH=1 ${pass_ndctl_enable} SRCVERSION=$SRCVERSION
+	dh_auto_install -- EXPERIMENTAL=${EXPERIMENTAL} prefix=/$PREFIX libdir=/$LIB_DIR includedir=/$INC_DIR docdir=/$DOC_DIR man1dir=/$MAN1_DIR man3dir=/$MAN3_DIR man5dir=/$MAN5_DIR man7dir=/$MAN7_DIR sysconfdir=/etc bashcompdir=/usr/share/bash-completion/completions NORPATH=1 ${pass_ndctl_enable} SRCVERSION=$SRCVERSION PMEM2_INSTALL=${PMEM2_INSTALL}
 	find -path './debian/*usr/share/man/man*/*.gz' -exec gunzip {} \;
 
 override_dh_install:
@@ -600,7 +619,7 @@ cat << EOF > debian/libpmem-dev.lintian-overrides
 $ITP_BUG_EXCUSE
 new-package-should-close-itp-bug
 # The following warnings are triggered by a bug in debhelper:
-# http://bugs.debian.org/204975
+# https://bugs.debian.org/204975
 postinst-has-useless-call-to-ldconfig
 postrm-has-useless-call-to-ldconfig
 # We do not want to compile with -O2 for debug version
@@ -637,7 +656,7 @@ cat << EOF > debian/libpmemblk-dev.lintian-overrides
 $ITP_BUG_EXCUSE
 new-package-should-close-itp-bug
 # The following warnings are triggered by a bug in debhelper:
-# http://bugs.debian.org/204975
+# https://bugs.debian.org/204975
 postinst-has-useless-call-to-ldconfig
 postrm-has-useless-call-to-ldconfig
 # We do not want to compile with -O2 for debug version
@@ -673,7 +692,7 @@ cat << EOF > debian/libpmemlog-dev.lintian-overrides
 $ITP_BUG_EXCUSE
 new-package-should-close-itp-bug
 # The following warnings are triggered by a bug in debhelper:
-# http://bugs.debian.org/204975
+# https://bugs.debian.org/204975
 postinst-has-useless-call-to-ldconfig
 postrm-has-useless-call-to-ldconfig
 # We do not want to compile with -O2 for debug version
@@ -716,7 +735,7 @@ cat << EOF > debian/libpmemobj-dev.lintian-overrides
 $ITP_BUG_EXCUSE
 new-package-should-close-itp-bug
 # The following warnings are triggered by a bug in debhelper:
-# http://bugs.debian.org/204975
+# https://bugs.debian.org/204975
 postinst-has-useless-call-to-ldconfig
 postrm-has-useless-call-to-ldconfig
 # We do not want to compile with -O2 for debug version
@@ -752,7 +771,7 @@ cat << EOF > debian/libpmempool-dev.lintian-overrides
 $ITP_BUG_EXCUSE
 new-package-should-close-itp-bug
 # The following warnings are triggered by a bug in debhelper:
-# http://bugs.debian.org/204975
+# https://bugs.debian.org/204975
 postinst-has-useless-call-to-ldconfig
 postrm-has-useless-call-to-ldconfig
 # We do not want to compile with -O2 for debug version
@@ -763,77 +782,6 @@ hardening-no-fortify-functions $LIB_DIR/pmdk_dbg/*
 libpmempool-dev: package-has-unnecessary-activation-of-ldconfig-trigger
 EOF
 
-cat << EOF > debian/libvmem.install
-$LIB_DIR/libvmem.so.*
-EOF
-
-cat << EOF > debian/libvmem.lintian-overrides
-$ITP_BUG_EXCUSE
-new-package-should-close-itp-bug
-libvmem: package-name-doesnt-match-sonames
-EOF
-
-cat << EOF > debian/libvmem-dev.install
-$LIB_DIR/pmdk_debug/libvmem.a $LIB_DIR/pmdk_dbg/
-$LIB_DIR/pmdk_debug/libvmem.so	$LIB_DIR/pmdk_dbg/
-$LIB_DIR/pmdk_debug/libvmem.so.* $LIB_DIR/pmdk_dbg/
-$LIB_DIR/libvmem.so
-$LIB_DIR/pkgconfig/libvmem.pc
-$INC_DIR/libvmem.h
-$MAN7_DIR/libvmem.7
-$MAN3_DIR/vmem_*.3
-EOF
-
-cat << EOF > debian/libvmem-dev.lintian-overrides
-$ITP_BUG_EXCUSE
-new-package-should-close-itp-bug
-# The following warnings are triggered by a bug in debhelper:
-# http://bugs.debian.org/204975
-postinst-has-useless-call-to-ldconfig
-postrm-has-useless-call-to-ldconfig
-# We do not want to compile with -O2 for debug version
-hardening-no-fortify-functions $LIB_DIR/pmdk_dbg/*
-# pmdk provides second set of libraries for debugging.
-# These are in /usr/lib/$arch/pmdk_dbg/, but still trigger ldconfig.
-# Related issue: https://github.com/pmem/issues/issues/841
-libvmem-dev: package-has-unnecessary-activation-of-ldconfig-trigger
-EOF
-
-cat << EOF > debian/libvmmalloc.install
-$LIB_DIR/libvmmalloc.so.*
-EOF
-
-cat << EOF > debian/libvmmalloc.lintian-overrides
-$ITP_BUG_EXCUSE
-new-package-should-close-itp-bug
-libvmmalloc: package-name-doesnt-match-sonames
-EOF
-
-cat << EOF > debian/libvmmalloc-dev.install
-$LIB_DIR/pmdk_debug/libvmmalloc.a   $LIB_DIR/pmdk_dbg/
-$LIB_DIR/pmdk_debug/libvmmalloc.so   $LIB_DIR/pmdk_dbg/
-$LIB_DIR/pmdk_debug/libvmmalloc.so.* $LIB_DIR/pmdk_dbg/
-$LIB_DIR/libvmmalloc.so
-$LIB_DIR/pkgconfig/libvmmalloc.pc
-$INC_DIR/libvmmalloc.h
-$MAN7_DIR/libvmmalloc.7
-EOF
-
-cat << EOF > debian/libvmmalloc-dev.lintian-overrides
-$ITP_BUG_EXCUSE
-new-package-should-close-itp-bug
-# The following warnings are triggered by a bug in debhelper:
-# http://bugs.debian.org/204975
-postinst-has-useless-call-to-ldconfig
-postrm-has-useless-call-to-ldconfig
-# We do not want to compile with -O2 for debug version
-hardening-no-fortify-functions $LIB_DIR/pmdk_dbg/*
-# pmdk provides second set of libraries for debugging.
-# These are in /usr/lib/$arch/pmdk_dbg/, but still trigger ldconfig.
-# Related issue: https://github.com/pmem/issues/issues/841
-libvmmalloc-dev: package-has-unnecessary-activation-of-ldconfig-trigger
-EOF
-
 cat << EOF > debian/$PACKAGE_NAME-dbg.lintian-overrides
 $ITP_BUG_EXCUSE
 new-package-should-close-itp-bug
@@ -869,6 +817,13 @@ then
 	rpmem_install_triggers_overrides;
 fi
 
+# libpmem2
+if [ "${PMEM2_INSTALL}" == "y" ]
+then
+	append_libpmem2_control;
+	libpmem2_install_triggers_overrides;
+fi
+
 # daxio
 if [ "${NDCTL_ENABLE}" != "n" ]
 then
@@ -876,7 +831,6 @@ then
 	daxio_install_triggers_overrides;
 fi
 
-
 # Convert ChangeLog to debian format
 CHANGELOG_TMP=changelog.tmp
 dch --create --empty --package $PACKAGE_NAME -v $PACKAGE_VERSION-$PACKAGE_RELEASE -M -c $CHANGELOG_TMP
